function [arcLength] = computeArcLength(p,firstPoint,lastPoint)
% [arcLength] = computeArcLength(p,firstPoint,lastPoint)
%
% Computes length of a second-order curve between two points
%
% Input:
%   p           - array of curve coefficients (p1,p2,p3), which define 
%                 the curve as y(x) = p1*x^2 + p2*x + p3
%   firstPoint  - point in format (x,y)
%   lastPoint   - point in format (x,y)
%
% Output:
%   arclength   - arc length of the curve between firstPoint and lastPoint
%
% Last modified: 2017-07-03, Spartak Zikrin

if abs(p(1)) > 1e-10
    % Transform coordinates and compute the analytical solution of 
    % the integral of sqrt(1+x^2)
    transfPoints = sign(p(1))*(2*p(1)*[firstPoint; lastPoint] + p(2));
    sqrtExp = sqrt(transfPoints.^2+1);
    arcLength =  diff(transfPoints.*sqrtExp + log(transfPoints+sqrtExp)) / ...
                 (4*abs(p(1)));
else
    % Fit curve is (almost) a straight line
    fitPoints = p(2)*[firstPoint; lastPoint] + p(3);
    arcLength = sqrt((firstPoint-lastPoint)^2 + (fitPoints(1)-fitPoints(2))^2);
end

end