function computeParticleInternalCoord(mCells)
% computeParticleInternalCoord(mCells)
%
% This function computes coordinates of the detected cell particles in the
% cell internal coordinate system.
%
% Input:
%   mCells - array of MCell objects

% Last modified: 
% 2018-05-04: bugfix, cellDetectionFrames is a vertical array and must be
% transposed to use in the for-loop.
% 2017-12-18, SZ.
% - Moved away computation of phase coordinates to Spot.processDots.m
% - Bugfix when cell measurement fails, the NAN values for internal 
%   coordinates were not previously saved

import Cell.Measure.convertToInternalCoord

for cIdx = 1:length(mCells)
    c = mCells(cIdx);
    for pIdx = 1:length(c.particles)
        p = c.particles(pIdx);
        for detectionFrame = p.cellDetectionFrames'
            t = c.getIndex(detectionFrame);
            if c.widths(t) <= 0 || c.lengths(t) <= 0
                outNaN = repmat([NaN NaN], size(p.trajImageCoord,1),1);
                c.particles(pIdx).trajInternalCoord = outNaN;
            else                
                convertToInternalCoord(c,pIdx,t);
            end
        end
    end
end