function outImage = cropCell(inputImage, aCellTrack, trackPt)
% outImage = cropCellImage(inputImage, aCellTrack, trackPt)
% 
% This function crops an subimage from the phase contrast image with
% a cell.
%
% Inputs
%   inputImage      : Phase image with segmented cells
%   aCellTrack      : Object containing info about a tracked cell
%   trackPt         : integer scalar, point in the cell track
%
% Output
%   outImage        : Image that contains cell cropped from inputImage

bb = aCellTrack.boundingBoxes(:,trackPt);
blobLabel = aCellTrack.blobLabels(trackPt);
outImage = inputImage(bb(2):bb(2)+bb(4)-1,bb(1):bb(1)+bb(3)-1) == blobLabel;

if aCellTrack.isRotated(trackPt)
    outImage = rot90(outImage);
end

end