function filteredScores  = removeLowMigrationScores(migrationScores,splitScores,mergeScores,parameters)
% Detects migration scores below threshold defined by
% migJaccardPostSplitMin and if zero probability of cell split/merge,
% remove the score.
%
% Last modified: Spartak Zikrin, 2018-01-12

if isempty(migrationScores)
    filteredScores = [];
    return
end

cutOffValue = parameters.migJaccardPostSplitMin;
lowScoreIndices = find(migrationScores(:,5)' < log(cutOffValue));

delIndex = false(size(migrationScores,1),1);
for i = lowScoreIndices
    t = migrationScores(i,1);
    
    cellSplits = splitScores(:,1) == t;
    if any(cellSplits) && any(splitScores(cellSplits, 2) == migrationScores(i,2))
        continue
    end

    cellMerges = mergeScores(:,1) == t;
    if any(cellMerges) && any(mergeScores(cellMerges, 2) == migrationScores(i,3))
        continue
    end
    
    delIndex(i) = true;
end

filteredScores = migrationScores(~delIndex,:);

end