function similarityIndex = computeJaccardIndex(blob1, blob2)
% similarityIndex = computeJaccardIndex(blob1, blob2)
% 
% Computes the Jaccard similarity index between two blobs.
%
% The Jaccard similarity index is the intersection of the binary masks,
% divided by the union of the binary masks. The Jaccard similarity index
% can be used as a measure of segmentation accuracy and as a feature or
% likelihood for classification of migration events. The measure is
% symmetric in the two input blobs.
%
% Inputs:
% blob1 - First blob, a structure containing image bounding box, 
%         boundary (as pixel locations) and blob area.
% blob2 - Second blob
%
% Outputs:
% similarityIndex - Jaccard similarity index for the binary masks of blob1
%                   and blob2.
%           
%
% See also:
% MigLogLikeList_Jaccard
%
% Comments are up to date.

% Compute intersection of the blobs
import Cell.Track.Curate.computeBlobsOverlap

intersection = computeBlobsOverlap(blob1, blob2);
if ~intersection
    similarityIndex = 0;
else
    union = blob1.area + blob2.area - intersection;
    similarityIndex = intersection / union;
end

end