function [outMCells] = concatenateCells(mCells, cellPairs)
% [outCells] = concatenateCells(mCells, cellPairs)
% 
% Concatenate  pairs of disappearing and appearing after a frame cell 
% tracks into a single cell track with a gap. 
% The new cell is labeled as follows:
% isBadCell = 4 and badSegmentation = 4 in the missing frame 
%
% Inputs:
%   mCells - array of MCell objects representing tracked cells.
%   cellPairs - 2xn array where the first row is the indices of
%               disappearing cells, and the second row is the indices of
%               appearing cells, correspondingly.% See findTrackGaps.m
%
%   outMCells - array of MCell objects with remaining tracked cells after
%              track concatenation

% Last modified: SZ, 2018-06-13. Handle the case of empty mCells.

if isempty(cellPairs)
    outMCells = mCells;
    return;
end

disappFrames = [mCells(cellPairs(1,:)).lastFrame];
appFrames = [mCells(cellPairs(2,:)).birthFrame];

if any(appFrames-disappFrames ~= 2)
    disp('Wrong frame gap between cell tracks');
    outMCells = mCells;
    return;
end

[~, ind] = sort(disappFrames,'descend');
cellPairs = cellPairs(:,ind);

for i=1:size(cellPairs,2)
    cDisapp = mCells(cellPairs(1,i));
    cApp = mCells(cellPairs(2,i));
    cDisapp.addCellWithGap(cApp);
    cApp.deleteCell();
end

isCellDeleted = false(1,numel(mCells));
isCellDeleted(cellPairs(2,:)) = true;
outMCells = mCells(~isCellDeleted);

for cIdx = 1:numel(outMCells)
    outMCells(cIdx).id = cIdx;
end