function outMCells = removeShortTracks(mCells, trackLengthCutOff)
% removeShortTracks(mCells, trackLengthCutOff, keepMultiGen)
%
% This function deletes cells that have too few points of detection. 
% Cell trajectories which have parents and/or daugthers are not
% removed even if they are below trackLengthCutOff.
%
% Input: 
%   mCells            - array of MCell objects representing tracked cells
%   trackLengthCutOff - Defines the shortes length of trajectories to
%                       keep.
%
% Output:
%   outMCells         - array of MCells objects with trajectory length
%                       above the threshold

% Last modified: Spartak Zikrin, 2018-03-05. Always keep cells with lineage

delIdx = false(length(mCells),1);
for ii = 1:length(mCells)
    c = mCells(ii);
    if isempty(c.parent) && isempty(c.descendants) && c.lifeTime < trackLengthCutOff
        c.deleteCell();
        delIdx(ii) = true;
    end
end

outMCells = mCells(~delIdx);

for ii = 1:length(outMCells)
    outMCells(ii).id = ii;
end