function outMCells = curateTrackingOutput(mCells, parameters)
% outMCells = curateTrackingOutput(mCells, parameters)
% 
% This function is wrapper for calling several different
% sub-functions which label cells, function that concatenates pairs of
% disappearing and appearing after a frame cells, and a function that
% deletes short cell tracks.
% 
% The three sub-functions are:
%  labeling of mis-segmentaions
%  deletion of short tracks.
%  finding cells with cells sharing the same segmentation blob.
%
% We try to keep these functions as close to the
% Baxter output as possible. This way we allow for 
% flexibility and possiblity of sharing the code. 
%
% Input:
%   mCells      - array of MCell objects representing tracked cells
%   parameters  - sruct with parameters
%
% Output:
%   outMCells   - resulting array of MCell objects

% Last modified: Spartak Zikrin, 2018-03-05. Changed detection of
% missegmentation, see findAndLabelMisSegmentation
             
import Cell.Track.Curate.*

% Parse parameters
jacCutOff = parameters.JacMigCutOff;
comCutOff = parameters.centerOfMassMovementCutOff;
trackCutOff = parameters.shortTrackCutOff;
maxGrowthRate = parameters.maxGrowthRate;

mFrames = Cell.MCell.mFrames;
findAndLabelConvergingTracks(mFrames);
findAndLabelMisSegmentation(mCells, maxGrowthRate);
findAndLabelRapidCMMovements(mCells, comCutOff);
nFrames = mFrames.getNumOfFrames();
cellPairs = findDisappearAppearCells(mCells, nFrames, jacCutOff, comCutOff);
catMCells = concatenateCells(mCells, cellPairs);
outMCells = removeShortTracks(catMCells, trackCutOff);

end