function mCells = mergeCellTracksInTraps(arrayOfMCellArrays, nFrames)
% mCells = mergeCellTracksInTraps(arrayOfMCellArrays, nFrames)
% 
% Input:
%   arrayOfMCellArrays - cell array, where each cell contains an array of
%   MCell objects representing cell tracks in the corresponging trap
%   nFrames - number of phase frames
%
% Output:
%   mCells - array of MCell objects representing cell tracks in the whole
%   sequence of frames.

nChannels = length(arrayOfMCellArrays);
for ch = 1:nChannels
    for i=1:length(arrayOfMCellArrays{ch})
        arrayOfMCellArrays{ch}(i).growthChannel = ch;
    end
end

L = [0; cumsum(cellfun('length',arrayOfMCellArrays))];
nCells = L(end);
mCells(1:nCells,1) = Cell.MCell;
for ch = 1:nChannels
    if ~isempty(arrayOfMCellArrays{ch})
        mCells(L(ch)+1:L(ch+1)) = arrayOfMCellArrays{ch};
    end
end
for i=1:nCells
    mCells(i).id = i;
end

mFrames = Cell.MCell.mFrames;
mFrames.initialize(nFrames);
M = zeros(nCells,nFrames);
for i=1:nCells
    c = mCells(i);
    M(i,c.birthFrame:c.lastFrame) = c.blobLabels;
end
for fIdx = 1:nFrames
    cellIndices = find(M(:,fIdx));
    blobLabels = M(cellIndices,fIdx);
    [keySet, ~, keyIndices] = unique(blobLabels);
    valueSet = cell(1,length(keySet));
    for ii = 1:length(cellIndices)
        bIdx = keyIndices(ii);
        cIdx = cellIndices(ii);
        valueSet{bIdx} = [valueSet{bIdx}, mCells(cIdx)];
    end
    mFrames.setPointersInFrame(fIdx,keySet,valueSet);
end