function [logLobj, fluoOffset] = getLogL_EMCCD_LookupTable(expInfoObj,fluoChanName)

sourceDir = expInfoObj.getPathName('source');
outputDir = expInfoObj.getPathName('output');
positionList = expInfoObj.getPositionList();

parameters = expInfoObj.getParameters();
EMCCD_offsetName = parameters.EMCCD_offsetName;
EMgain = parameters.EMgain;
EMCCD_noiseStd = parameters.EMCCD_noiseStd;
nCores = parameters.parNumWorkers;

if exist(EMCCD_offsetName,'file')
    calibrationOffsetFile = EMCCD_offsetName;
else
    calibrationOffsetFile = fullfile(sourceDir,EMCCD_offsetName);
end
fluoOffset=double(imread(calibrationOffsetFile)); % the average pixel offset, converted do double

saveName = fullfile(outputDir,'logLobj.mat');
if exist(saveName,'file') 
    tmp = load(saveName,'logLobj','fluoOffset');
    if tmp.logLobj.EMgain==EMgain && tmp.logLobj.sigmaRead==EMCCD_noiseStd && ...
            isequal(tmp.fluoOffset,fluoOffset)
        logLobj = tmp.logLobj;
        return
    end
end

offsetMin=min(fluoOffset(:));
outData = zeros(size(positionList));
parfor (posIdx = 1:length(positionList), nCores)
    currPos = positionList{posIdx};
    fluoDir = fullfile(sourceDir,currPos,fluoChanName);
    fluoList = dir(fullfile(fluoDir,'*.tif*'));
    imRangeFluo = expInfoObj.getRange(currPos,fluoChanName);
    cMaxTmp=-inf;
    for idx=1:length(imRangeFluo)
        fluoIdx = imRangeFluo(idx);
        currFluoMovieFile=fullfile(fluoDir,fluoList(fluoIdx).name);
        % movie size
        FlInfo = imfinfo(currFluoMovieFile);
        nFrames= numel(FlInfo);

        MV=tiffread(currFluoMovieFile,1);  % first frame is usually brightest
        cMaxTmp=double(max(cMaxTmp,max(MV(:))-offsetMin));
        
        if nFrames > 1
            MV=tiffread(currFluoMovieFile,nFrames); % last frame is usually dimmest
            cMaxTmp=double(max(cMaxTmp,max(MV(:))-offsetMin));
        end
    end
    outData(posIdx) = cMaxTmp;
end
cMax = max(outData);

logLobj = EMCCDfit.logL_EMCCD_lookup(EMgain,EMCCD_noiseStd,cMax,cMax/EMgain*3,nCores);

saveName = fullfile(outputDir,'logLobj.mat');
save(saveName,'logLobj','fluoOffset');