function [cropROI, subIm] = cropRectangle(im)
% [cropROI, subIm] = cropRectangle(im)
%
% Crops a rectangle from image im which is a result of another image
% transformation and hence represents a rotated rhombus

% Last modified: 2018-04-20, SZ. Added an offset parameter (flex) to handle
% the case of minor rotations
flex = 5;
[fRow, ~] = find(im>0,1);
lColArray = zeros(flex,1);
for idx=1:flex
    lColArray(idx) = find(im(fRow+idx-1,:)>0,1,'last');
end
[lCol,idx] = max(lColArray);
fRow = fRow+idx-1;
rowsArray = zeros(flex,2);
for idx=1:flex
    rowsArray(idx,1) = find(im(:,lCol-idx+1)>0,1,'first');
    rowsArray(idx,2) = find(im(:,lCol-idx+1)>0,1,'last');
end
[~,idx] = max(rowsArray(:,2)-rowsArray(:,1));
lCol = lCol-idx+1;
row1 = rowsArray(idx,1);
row2 = rowsArray(idx,2);

if fRow-row1 < row2-fRow
    lRow = row2;
    fColArray = zeros(flex,1);
    for idx=1:flex
        fColArray(idx) = find(im(lRow-idx+1,:)>0,1,'first');
    end
    [fCol, idx] = min(fColArray);
    lRow = lRow-idx+1;
else
    lRow = fRow;
    fRow = row1;
    fColArray = zeros(flex,1);
    for idx=1:flex
        fColArray(idx) = find(im(fRow+idx-1,:)>0,1,'first');
    end
    [fCol, idx] = min(fColArray);
    fRow = fRow+idx-1;
end

cropROI = [fCol fRow lCol-fCol lRow-fRow];

if nargout == 2
    subIm = imcrop(im,cropROI);
end