function [emptyChannelIdx, emptyChannelPixelSum] = findEmptyChannel(im,channelLocations)
% Finds an empty channel in the phase contrast image
% 
% Input:
%   im                   - phase contrast image of the mother machine.
%   channelLocation      - array of locations of channels' centerlines, 
%                          in pixels.
%
% Output:
%   emptyChannelIdx      - index of the empty channel.
%   emptyChannelPixelSum - sum of image pixel values along the 
%                          centerline of the empty channel.

if isnan(channelLocations)
    emptyChannelIdx = NaN;
    emptyChannelPixelSum = NaN;
    return
end

pixelSumPerChannel = sum(im(channelLocations,:),2);

[emptyChannelPixelSum, emptyChannelIdx] = max(pixelSumPerChannel);

end
