classdef aCell < handle
    % A cell which contains tracking info, cell info, particles info and
    % lineage
    
    properties
        cellId              = [];
        birthFrame          = [];
        parent              = [];
        descendants         = [];
        cellLabels          = [];
        fluoMovieIdx        = [];
        backBoneFit         = [];
        isRotated           = [];
        cellLength          = [];
        cellWidth           = [];
        majorAxisOrientationDegree= []; 
        fluoToSegScaling    = [];
        phaseTimes          = [];
        particles           = struct('channelName',       [], ...
                                     'trajImageCoord',    {}, ...
                                     'trajImageCov',      {}, ...
                                     'trajInternalCoord', {}, ...
                                     'numberOfFrame',     [], ...
                                     'firstFrame',        [], ...
                                     'fluoTimes',         [], ...
                                     'localisationError', [], ...
                                     'extraInfo',         []);
    end
    
    methods
        % Constructor
        function this = aCell(varargin)
            for i = 1 : 2 : length(varargin)
                this.(varargin{i}) = varargin{i+1};
            end
        end
        
        function oCell = Clone(this)
            % Returns a deep copy of a cell object.
            %
            % Normal assignment statements only copies the pointer to the
            % cell object.
            
            oCell = aCell('cellId',             this.cellId, ...
                          'birthFrame',         this.birthFrame, ...
                          'parent',             this.parent, ...
                          'descendants',        this.descendants, ...
                          'cellLabels',         this.cellLabels, ...
                          'fluoMovieIdx',       this.fluoMovieIdx, ...
                          'backBoneFit',        this.backBoneFit, ...
                          'isRotated',          this.isRotated, ...
                          'cellLength',         this.cellLength, ...
                          'cellWidth',          this.cellWidth, ...
                          'metadataPath',       this.metadataPath, ...
                          'fluoToSegScaling',   this.fluoToSegScaling, ...
                          'phaseTimes',         this.phaseTimes, ...
                          'particles',          this.particles);
        end
        
        % Other functions
        function oLifetime = getLifetime(this)
            if size(this,2) == 1
                oLifetime = length(this.cellLabels);
            else
                oLifetime = zeros(1,size(this,2));
                for cellIdx = 1:size(this,2)
                    oLifetime(cellIdx) = length(this(cellIdx).cellLabels);
                end
            end
        end
        
        function oCentroid = getCentroid(this, frameIdx, segIm)
            if size(this,2) == 1
                currLabelIm = segIm == this.cellLabels(frameIdx);
                res = regionprops(currLabelIm,'centroid');
                oCentroid = res.Centroid;
            else
                disp('getCentroid processes only a single cell in a single frame'); 
            end
        end
        
        function oCellVolume = getCellVolume(this)
            if size(this,2) == 1
                oCellVolume = zeros(length(this.cellLength));
                for frameIdx = 1:length(this.cellLength)
                    cylinderVolume = this.cellLength(frameIdx)*pi*((this.cellWidth(frameIdx)/2)^2);
                    sphereVolume = (4/3)*pi*((this.cellWidth(frameIdx)/2)^3);
                    oCellVolume(frameIdx) = cylinderVolume + sphereVolume;
                end
            else
                for cellIdx = 1:size(this,2)
                    for frameIdx = 1:length(this.cellLength)
                        cylinderVolume = this.cellLength(frameIdx)*pi*((this.cellWidth(frameIdx)/2)^2);
                        sphereVolume = (4/3)*pi*((this.cellWidth(frameIdx)/2)^3);
                        oCellVolume{cellIdx,frameIdx} = cylinderVolume + sphereVolume;
                    end
                end
            end
        end
    end
end
