function [refinedDots, MAPcoord, MAPcov, MAPpar] = MAPsgS0refineDotsSingleMovie(fluoMovieFile,fluoStackRange,movieDots,fluoOffset,logLobj,PSFsymGauss,parameters,nCores,printInfo)

if ~exist('nCores','var') || isempty(nCores)
    nCores = 0;
end

if ~exist('printInfo','var') || isempty(printInfo)
    printInfo = 0;
end

if isempty(movieDots)
    refinedDots=[];
    MAPcoord=[];
    MAPcov=[];
    MAPpar=[];
    return
end
%% Extract the required parameters
ROIwidth=parameters.LSQrefineDots_ROIwidth; % ROI side length for MLE fits
SminPx= parameters.LSQrefineDots_SminPx;    % minimum lsq psf width, in fluo pixel units
SmaxPx= parameters.LSQrefineDots_SmaxPx;    % maximum lsq psf width, in fluo pixel units
drMax = parameters.LSQrefineDots_drMax;     % maximum difference between centroid and lsq positions, in fluo pixel units
Nmin  = parameters.LSQrefineDots_Nmin;      % minimum spot amplitude (~photons, if offset&EMgain are correct)
Nquad = parameters.pxQuadPoints;
%% Main loop
import EMCCDfit.refineSingleFrame

MAPcoord=movieDots;
MAPcov=zeros(size(MAPcoord,1),6);
MAPpar=zeros(size(MAPcoord,1),3);

if nCores<=1
    for frameIdx = 1:length(fluoStackRange)
        frame=fluoStackRange(frameIdx);
        %%%warning('restricted number of frames!') % for debugging,
        %%% restricting the frame-loop can save lots of time... /ML            
        ind = (movieDots(:,3) == frame);
        if any(ind)
            frameIm = double(tiffread(fluoMovieFile,frame));
            frameCoord=movieDots(ind,1:2);
            [frameCoordMAP,dotCovMAP,dotParam,~,frameParMAP]=...
                refineSingleFrame(frameCoord,frameIm,fluoOffset,ROIwidth,Nquad,logLobj,PSFsymGauss,[],false);
            MAPcoord(ind,1:2) = frameCoordMAP;
            MAPcov(ind,:) = [frameParMAP [dotParam.logL]' [dotParam.opt_converged]' [dotParam.tFit]' ];
            MAPpar(ind,:) = dotCovMAP;
        end
    end
else
    dotFrames = movieDots(:,3);
    outDotsMAP = cell(1,length(fluoStackRange));
    outDotParMAP = cell(1,length(fluoStackRange));
    outDotCovMAP = cell(1,length(fluoStackRange));
    inFrameCooords = cell(1,length(fluoStackRange));
    for frameIdx = 1:length(fluoStackRange)
        frame=fluoStackRange(frameIdx);
        inFrameCooords{frameIdx} = movieDots(dotFrames == frame,1:2);
    end
    parfor (frameIdx = 1:length(fluoStackRange), nCores) 
        frame=fluoStackRange(frameIdx);         
        ind = (dotFrames == frame);
        if any(ind)
            frameIm=double(tiffread(fluoMovieFile,frame));
            frameCoord=inFrameCooords{frameIdx};
            [frameCoordMAP,dotCovMAP,dotParam,~,frameParMAP]=...
                refineSingleFrame(frameCoord,frameIm,fluoOffset,ROIwidth,Nquad,logLobj,PSFsymGauss,[],false);
            outDotsMAP{frameIdx} = frameCoordMAP;
            outDotParMAP{frameIdx} = [frameParMAP [dotParam.logL]' [dotParam.opt_converged]' [dotParam.tFit]' ];
            outDotCovMAP{frameIdx} = dotCovMAP;
        end
    end
    for frameIdx = 1:length(fluoStackRange)
        frame=fluoStackRange(frameIdx);
        ind = (movieDots(:,3) == frame);
        MAPcoord(ind,1:2) = outDotsMAP{frameIdx};
        MAPcov(ind,:) = outDotParMAP{frameIdx};
        MAPpar(ind,:) = outDotCovMAP{frameIdx};
    end
end
%% Filter out positions based on convergence, spot size, and
% fitted position (not) straying too far from the ROI center
S=MAPcov(:,3);
N=MAPcov(:,2);
flag=MAPcov(:,5);

dx=MAPcoord(:,1)-movieDots(:,1);
dy=MAPcoord(:,2)-movieDots(:,2);
dr=sqrt(dx.^2+dy.^2);
MAPcov(:,7)=dr;

xVar=MAPpar(:,1);
yVar=MAPpar(:,3);
covDet=MAPpar(:,1).*MAPpar(:,3)-MAPpar(:,2).^2;
covTrace=MAPpar(:,1)+MAPpar(:,2);   
% covDet>0 and covTrace>0 means both eigenvalues are positive, i.e.,
% covariance matrix is positive definite and thus variances are
% positive in all coordinate systems. 

ind = (dr<drMax) & (S>SminPx) & (S<SmaxPx) & (flag==true) & ...
    (N>Nmin) & (xVar>0) & (yVar>0) & (covDet>0) & (covTrace>0);
MAPcov(:,8)=0;
MAPcov(ind,8)=1;

tFit=mean(MAPcov(:,6));
MAPfraction=sum(ind)/size(movieDots,1);
if printInfo
    disp('--------------------')
    disp(['symMAP fitted ' int2str(sum(ind)) ' dots from ' fluoMovieFile ' .'])
    disp(['Average symMAP fit time    : ' num2str(tFit,4) ' s/spot'])
    disp(['symMAP filter kept ' num2str(MAPfraction*100) ' % of fitted dots.'])
    disp('--------------------')
end

% put the final answer back in the cell vector
refinedDots = [MAPcoord(ind,:) MAPpar(ind,1:3)];