function [rowInd,colInd,x0,y0]=ROItransform(x_c,y_c,x_w,y_w,imSize)
% [rowInd,colInd,x0,y0]=ROItransform(x_c,y_c,x_w,y_w,imSize)
%
% Compute index range and coordinate transform for a region of interest
% (ROI) in an image.
%
% input
% x_c,y_c   : ROI center, in pixel coordinates
% x_w,y_w   : ROI width in the x and y directions (pixels)
% imSize    : Size of the parent image, as in imSize=size(parentImage).
%             (Only the first two indices are used).
%
% Output
% rowInd,colInd   : The region of interest can be plotted by, e.g., 
%                   imagesc(parentImage(rowInd,colInd). The index ranges
%                   are truncated to be in the allowed range of the parent
%                   image.
% x0,y0 : coordinate transform between parent and ROI.
%         x_parent = x_ROI+x0, y_parent=y_ROI+y0.
%
% Martin Lindén, bmelinden@gmail.com, 2015-11-05
x_min=max(1        , ceil(x_c-x_w/2));
x_max=min(imSize(2),floor(x_c+x_w/2));

y_min=max(1        , ceil(y_c-y_w/2));
y_max=min(imSize(1),floor(y_c+y_w/2));

rowInd=y_min:y_max;
colInd=x_min:x_max;

x0=x_min-1;
y0=y_min-1;


