function [totalCellFluo,cellArea]=cellBleachingCurves(sourceFolder,fluoChannelName, fluoList, pathSegImages, segImagesList,segDilation,EMCCD_offsetName)
% [totalCellFluo,cellArea]=cellBleachingCurves(...
%    sourceFolder,fluoChannelName, fluoList, pathSegImages, segImagesList,segDilation,EMCCD_offsetName)
% extract total cell fluorescence curves, and the cell area (number of fluo
% pixels) used to compute the curves, so that conversion to average
% fluorescence per pixel is possible. 
% input:
% sourceFolder          : as in the image analysis pipeline
% fluoChannelName      : 
%
% output: 
% totalCellFluo{m}{k} : total fluorescence of cell k in movie m
% cellArea{k}(k)      : number of pixels used to compute totalCellFluo{m}{k}
%
% ML 2015-11-24

if(~exist('segDilation','var') || isempty(segDilation) )
    segDilation=0;
end

% loop over movies
maxY=0;
cellArea=cell(1,length(fluoList));
numSpots=cell(1,length(fluoList));
maxNumLater=cell(1,length(fluoList));

totalCellFluo=cell(1,length(fluoList));

parfor m = 1:length(fluoList)
%%%for m = 1:length(fluoList)
    tm0=tic;
    % loop over cells
    currSegmentedImage = imread(fullfile(pathSegImages,segImagesList(m).name));
    cMax=double(max(currSegmentedImage(:)));
    
    fluoFile=fullfile(sourceFolder,fluoChannelName,fluoList(m).name);
    IMF=imfinfo(fluoFile);
    frames=1:length(IMF);   
    
    currFluoMovie=BuildTrajectories.ML_loadStack2(fluoFile);
    totalCellFluo{m}=cell(1,cMax);
    cellArea{m}=zeros(1,cMax);
        
    % EMCCD offset image
    IM0=BuildTrajectories.ML_loadStack2(fullfile(sourceFolder,EMCCD_offsetName));
    for c=1:cMax;
        
        % sum up total cell fluorescence, including an extra
        % boundary pixel to counter roundoff artifactss
        se=strel('disk',segDilation);
        fluoCellMask=imresize(imdilate(currSegmentedImage==c,se),...
            [IMF(1).Height IMF(1).Width],'nearest');
        cellArea{m}(c)=sum(sum(fluoCellMask));
        totalCellFluo{m}{c}=zeros(size(frames));
        for t=frames
            %currFluoFrame=imread(fluoFile,t,'info',IMF); % ML: very slow
            currFluoFrame=currFluoMovie(:,:,t);
            %totalCellFluo{m}{c}(t)=sum(sum(currFluoMovie(:,:,t).*fluoCellMask));
            %%ML: very slow
            totalCellFluo{m}{c}(t)=sum(currFluoFrame(fluoCellMask))-sum(IM0(fluoCellMask));
        end
    end
    disp(['Cell bleaching curves in movie ' int2str(m) ', ' int2str(cMax) ' cells, ' num2str(toc(tm0)/60) ' min.'])
end
    

