% test script for PSF classes
clear
meta.abstractDetails('PSF.PSFmodel')

meta.abstractDetails('PSF.SymGaussS0')
meta.abstractDetails('PSF.SymGaussS0_MLE')
meta.abstractDetails('PSF.SymGaussS0_logNormBN_expS0')

meta.abstractDetails('PSF.AsymGaussS0')
meta.abstractDetails('PSF.AsymGaussS0_MLE')
meta.abstractDetails('PSF.AsymGaussS0_BNlnN_expS0')

% create an object and test functions
P=PSF.SymGaussS0_MLE('initialGuess',[3 3 log([1 200 0.1])],'lambda',639/80,'NA',1.4,'priorParameters',[]);
E=P.psfDensity(0,0,P.initialGuess);
[y,dy]=P.logPrior(P.initialGuess);
par0 = P.convertToOutStruct(P.initialGuess, struct)

P=PSF.SymGaussS0_logNormBN_expS0('initialGuess',[3 3 log([1 200 0.1])],'lambda',639/80,'NA',1.4,'priorParameters',[0 3 0 inf 1]);
E=P.psfDensity(0,0,P.initialGuess);
[y,dy]=P.logPrior(P.initialGuess);
par0 = P.convertToOutStruct(P.initialGuess, struct)

P=PSF.AsymGaussS0_MLE('initialGuess',[3 3 log([1 200 0.1 0.3]) 0],'lambda',639/80,'NA',1.4,'priorParameters',[]);
E=P.psfDensity(0,0,P.initialGuess);
[y,dy]=P.logPrior(P.initialGuess);
par0 = P.convertToOutStruct(P.initialGuess, struct)

P=PSF.AsymGaussS0_BNlnN_expS0('initialGuess',[3 3 log([1 200 0.1 0.3]) 0],'lambda',639/80,'NA',1.4,'priorParameters',[0 3 0 inf 1]);
E=P.psfDensity(0,0,P.initialGuess);
[y,dy]=P.logPrior(P.initialGuess);
par0 = P.convertToOutStruct(P.initialGuess, struct)

% log-prior derivatives



%% derivatives of intensity model
% this is a numerical test of the partial derivatives computed by
% PSF.psfDensity and PSF.logPrior. Note that these derivatives are not yet
% (2017-03-24) in use; that would require partial derivatives of the camera
% noise model as well, so they are not implemented in all classes yet.
k=1;
p0=P.initialGuess;
pk=p0(k)+linspace(-5,5,5000);
E=0*pk;
dEdpk=E;
xx=0.14;
yy=0.3;

for m=1:numel(pk)
    p1=p0;
    p1(k)=pk(m);
    [E(m),dEdp]= P.psfDensity(xx, yy, p1);
    dEdpk(m)=dEdp(k);
    [lnP0(m),dlnP0] = P.logPrior(p1);
    dlnP0dk(m)=dlnP0(k);
end


figure(1)
clf

subplot(2,1,1)
plot(pk,dEdpk,'linew',2)
hold on
plot((pk(1:end-1)+pk(2:end))/2,diff(E)./diff(pk),'r--','linew',2)


subplot(2,1,2)
plot(pk,dlnP0dk,'linew',2)
hold on
plot((pk(1:end-1)+pk(2:end))/2,diff(lnP0)./diff(pk),'r--','linew',2)




