function segMask = makeMaskFromSegmentedImage(segIm, fluoImSize, T)
% Function to take an image of segmented cells, make a mask
% containing all segment cells and then transform this mask into
% the size and resolution of a corresponding fluorescene image.
% Input parameters:
%   segIm - The segmented image in phase contrast.
%   fluoImSize - Size of fluorescence images. This is used to determine the
%            size of the final maks.
%   T - A transformation matrix of how to go from coordinates
%       of a fluorescense image to the coordinates of the 
%       corresponding phase contrast images.
    
invtfrom = invert(affine2d(T));

segMask = imwarp(segIm>0,invtfrom,'OutputView',imref2d(fluoImSize),'FillValues',0);