function [ filteredDots, nDotsPerFrame] = filterDotsInCell(cellDots,maxFrame,maxDots)
% [ filteredDots ] = filterDotsInCell(cellDots,maxFrame,maxDots)
%
% Count the number of dots in the cell per frame and filter out frames 
% with future dot number above some threshold
%
% Input:
%   cellDots - matrix, where each row corresponds to a dot and the first three
%   columns are [x y frame]
%   maxFrame - maximal fluo stack frame number
%   maxDots  - threshold, defines maximum allowed number of dots in a cell in each
%              frame

if isinf(maxDots) && nargout == 1
    filteredDots = cellDots;
    return;
end

nDotsPerFrame = zeros(1,maxFrame);
[frameNr,~,frameIdx] = unique(cellDots(:,3));
nDotsPerFrame(frameNr) = accumarray(frameIdx,1);

lastDiscardedFrame = find(nDotsPerFrame > maxDots,1,'last');
if ~isempty(lastDiscardedFrame)
    filteredDots = cellDots(cellDots(:,3)>lastDiscardedFrame,:);
else
    filteredDots = cellDots;
end

end

