import scipy as sp
import time
import sys
import os
import createbacteria  as cb
import numpy as np
import cv2
from scipy.ndimage import label
import cmath
import criticalpoints2
import string
import glob
import segmutils as su
from multiprocessing import Pool
    
#%###########################################################################
def weightFind((temparr,physicalSize)):
    uniq = np.unique(temparr)
    uniq = uniq[uniq>0]
    
    for ii in range(uniq.shape[0]):
        singlereg = temparr==uniq[ii]
        
        if( np.sum(singlereg)>physicalSize[5] or np.sum(singlereg)<physicalSize[4]  ):
            temparr[temparr==uniq[ii]] = 0
        else:
            
            params = cb.ellipseParam(np.int64(np.argwhere(singlereg==1)))                
            if( params[1] >physicalSize[0] and params[1]<physicalSize[1] and params[0]>physicalSize[2] and params[0] <physicalSize[3]): #choose this params accordingly on phase and fluorescent data             
                indivarea = np.sum(np.float32(singlereg))
                elipsarea = params[2]                
                                                        
                indivperim = np.sum(np.float32(singlereg - cv2.erode(np.uint8(singlereg) ,np.ones((3,3))) ))                    
                elipsperim = params[3]
                                                                        
                residual = np.abs(indivarea - elipsarea ) 
                residuarearatio = np.min([indivarea,  elipsarea] )/np.max([indivarea,  elipsarea] )
                    
                residuperimratio = np.min([indivperim,  elipsperim] )/np.max([indivperim,  elipsperim] )              
                convexity = indivarea/cb.findConvexarea(singlereg)
                weightval = 0.5*residuperimratio + 0.5*convexity
                temparr[temparr==uniq[ii]] = weightval
    
            else:
               temparr[temparr==uniq[ii]] = 0

    return(temparr)

#%###########################################################################
def bacmaintest(orimg,physicalSize,sigmaVal,weightThresh,nCores=12):   
 
    etemp = su.enhanceContrast(orimg, sigmaVal)
    
    #find histogram peak to detect start and end intensities
    histo, inten = np.histogram(etemp,range(256))
    
    histo = np.float32(histo)
    inten = np.float32(inten)
    
    meanval = np.int(np.sum((histo/histo.sum())*range(255)))
    
    peakval = inten[np.argwhere(histo == histo.max())[0][0]]
    startinten = peakval-45
    stopinten = peakval    
    
    LEVELS = np.int(stopinten-startinten)
    print "shape: ", etemp.shape
    print "levels: ", LEVELS
    labelimgarray = np.zeros((etemp.shape[0], etemp.shape[1], LEVELS), dtype=np.float)
    
    for ii in range(LEVELS):
        labelimgarray[:,:,ii], ncc = label((etemp>=(ii+startinten)),np.ones((3,3))) 
    
    weightarray = []
    sizeArray = []
    for ii in range(LEVELS):
        weightarray.append(np.float64(labelimgarray[:,:,ii]))
        sizeArray.append(physicalSize)

    if numCores>0:
        pool = Pool(numCores)
        results = pool.map(weightFind,zip(weightarray,sizeArray))
    else:
        results = []
        for ii in range(LEVELS):
            results.append(weightFind((weightarray[ii],sizeArray[ii])))
    
    maximg = np.max(results,0)
    maximg = sp.ndimage.binary_fill_holes(maximg>weightThresh)
    outimg , count = label(maximg)

    if numCores>0:
        pool.close()

    return(outimg)

#%###########################################################################
inputDir = sys.argv[1]
outputDir = sys.argv[2]
minWidth = int(sys.argv[3])
maxWidth = int(sys.argv[4])
minLength = int(sys.argv[5])
maxLength = int(sys.argv[6])
minArea = int(sys.argv[7])
maxArea = int(sys.argv[8])
weightThresh = float(sys.argv[9])
gaussKernSigma = float(sys.argv[10])
discardBorderCells = int(sys.argv[11])
numCores = int(sys.argv[12])

physicalSize = [minWidth, maxWidth, minLength, maxLength, minArea, maxArea];

inputDir = inputDir.replace('\\', '/')
	
outputDir = outputDir.replace('\\', '/')
if not outputDir.endswith('/'):
	outputDir = outputDir + '/'
if not os.path.isdir(outputDir):
	os.mkdir(outputDir)

if os.path.isfile(inputDir):
    inputDir, filename = os.path.split(inputDir)
    if inputDir:
        inputDir = inputDir + '/'
    files = []
    files.append(filename)
else:
    files = [name for name in os.listdir(inputDir) if os.path.isfile(os.path.join(inputDir, name))]

for i in range(0, len(files)):
    #starttime = time.time() 
    print(i)
    filename = files[i]
    inputImageName = inputDir + filename
    base, ext = filename.rsplit('.',1)
    outputImage = outputDir + base + "_segCBA." + ext

    orimg = cv2.imread(inputImageName,-1)
    res = bacmaintest(orimg,physicalSize,gaussKernSigma,weightThresh)

    if discardBorderCells:
        res = su.clearBorder(res,discardBorderCells)

    #print ' time ',  time.time()-starttime
    cv2.imwrite(outputImage,np.uint16(res))
