import PerObjectEllipsefit as poe
import numpy as np
import scipy
import scipy.ndimage as nd
#import time
import sys
import os
import cv2
import segmutils as su

#%###########################################################################
# Input parameters
inputDir = sys.argv[1]
outputDir = sys.argv[2]
minWidth = int(sys.argv[3])
maxWidth = int(sys.argv[4])
minLength = int(sys.argv[5])
maxLength = int(sys.argv[6])
minArea = int(sys.argv[7])
maxArea = int(sys.argv[8])
minEllipseRatio = float(sys.argv[9]);
maxEllipseRatio = float(sys.argv[10]);
ellipseFitThresh = float(sys.argv[11])
thresholdWindow = int(sys.argv[12])
gaussKernSigma = float(sys.argv[13])
discardBorderCells = int(sys.argv[14])

# Hardcoded parameters
brightObjects = True
resample = False
resampleMax = 255
clockTime = False
if clockTime:
    start = time.time()

inputDir = inputDir.replace('\\', '/')
	
outputDir = outputDir.replace('\\', '/')
if not outputDir.endswith('/'):
    outputDir = outputDir + '/'
if not os.path.isdir(outputDir):
    os.mkdir(outputDir)

if os.path.isfile(inputDir):
    inputDir, filename = os.path.split(inputDir)
    if inputDir:
        inputDir = inputDir + '/'
    files = []
    files.append(filename)
else:
    if not inputDir.endswith('/'):
        inputDir = inputDir + '/'
    files = [name for name in os.listdir(inputDir) if os.path.isfile(os.path.join(inputDir, name))]

for i in range(0, len(files)):	
    poeInstance = poe.PerObjectEllipsefit()
    filename = files[i]
    inputImageName = inputDir + filename
    base, ext = filename.rsplit('.',1)
    outputImage = outputDir + base + "_segPOE." + ext
    orimg = cv2.imread(inputImageName,-1)
    inputImage = su.enhanceContrast(orimg, gaussKernSigma)
    pixel_data = np.asarray(inputImage)

    if not brightObjects:
        maxp = np.amax(pixel_data)
        pixel_data[:] = [maxp - x for x in pixel_data]  

    if resample:
        maxp = np.amax(pixel_data)
        if maxp > resampleMax:
            scaleFactor = resampleMax/float(maxp)
            pixel_data[:] = [x * scaleFactor for x in pixel_data]
            pixel_data = pixel_data.astype(int)
			    
    #Extract objects
    res = poeInstance.run(pixel_data,minArea,maxArea,ellipseFitThresh,minLength,maxLength,minWidth,maxWidth,minEllipseRatio,maxEllipseRatio,thresholdWindow)

    #Fill holes
    res = nd.binary_fill_holes(res).astype(int)

    #Label objects
    (res, nObj) = nd.measurements.label(res)

    #Discard border cells
    if discardBorderCells:
        res = su.clearBorder(res,discardBorderCells)

    #Save image
    cv2.imwrite(outputImage,np.uint16(res))

if clockTime:
    elapsed = time.time() - start
    print("Total time: %s s" % str(elapsed))
