import torch
import sys
import os
import numpy as np
import math
from networks.basicUnet import basicUnet
from networks.Unetmod import Unetmod
from networks.nestedUnet import ShallowNestedUNet
from skimage import io
from skimage.measure import label, regionprops
from skimage.morphology import remove_small_objects
import segmutils as su
import time
#import matplotlib.pyplot as plt

#%###########################################################################
# Input parameters
inputDir = sys.argv[1]
outputDir = sys.argv[2]
modelLocation = sys.argv[3]

if len(sys.argv) >= 5:
    threshold = float(sys.argv[4])
else:
    threshold = 0.3

if len(sys.argv) >= 6:
    minCellArea = int(sys.argv[5])
else:
    minCellArea = 100

if len(sys.argv) == 7:
    discardBorderCells = int(sys.argv[6])
else:
    discardBorderCells = 1

start_test = time.time()

# Hardcoded parameters
deviceName = 'cpu' # or 'cuda:0'

#set the device
device = torch.device(deviceName)

#load trained network on Kuba data
saved_net_state = torch.load(modelLocation, map_location=device)
if saved_net_state['model_type'] == 'basicUnet':
    model_expansion = saved_net_state['model_expansion']
    net = basicUnet(model_expansion)
elif saved_net_state['model_type'] == 'Unetmod':
    model_expansion = saved_net_state['model_expansion']
    net = Unetmod(model_expansion)
elif saved_net_state['model_type'] == 'nestedUnet':
    #from networks.nestedUnet import ShallowNestedUNet
    net = ShallowNestedUNet(deep_supervision=True)
net.load_state_dict(saved_net_state['model_state_dict'])
net.to(device)
net.eval()

inputDir = inputDir.replace('\\', '/')

outputDir = outputDir.replace('\\', '/')
if not outputDir.endswith('/'):
    outputDir = outputDir + '/'
if not os.path.isdir(outputDir):
    os.mkdir(outputDir)

if os.path.isfile(inputDir):
    inputDir, filename = os.path.split(inputDir)
    if inputDir:
        inputDir = inputDir + '/'
    files = []
    files.append(filename)
else:
    if not inputDir.endswith('/'):
        inputDir = inputDir + '/'
    files = [name for name in os.listdir(inputDir) if os.path.isfile(os.path.join(inputDir, name))]

with torch.no_grad():
    for i in range(0, len(files)):
        filename = files[i]
        inputImageName = inputDir + filename
        base, ext = filename.rsplit('.',1)
        outputImageName = outputDir + base + "_segPN." + ext
        im = io.imread(inputImageName)
        im = im.astype('float32')
        im = (im-np.mean(im)) / np.std(im) # normalization
        im = torch.tensor(im).unsqueeze(0)

        #check if size if fit for neural net:
        sz = im.size()
        szBgrH = math.ceil(sz[1]/16)*16
        szBgrW = math.ceil(sz[2]/16)*16

        im2 = torch.from_numpy(np.zeros((1, 1, szBgrH, szBgrW), dtype=np.float32)).to(device)
        im2[0, 0, 0:sz[1], 0:sz[2]] = im

        # generate prediction mask
        net_pred = net(im2)
        net_pred = net_pred.to(deviceName).detach().numpy().squeeze(0).squeeze(0)
        net_pred = net_pred[0:sz[1], 0:sz[2]]
        mask = net_pred > threshold
        mask = remove_small_objects(mask,minCellArea)
        mask = mask.astype(int)
        res = label(mask)
        res = res.astype('uint16')

        if discardBorderCells:
            res = su.clearBorder(res,discardBorderCells)
    
        #save in designated folder with the same name
        io.imsave(outputImageName, res, check_contrast=False)
    
    end_test = time.time()
    print("segmentation done in: ", end_test-start_test)
    
        #plt.imshow(label_im)
        #plt.show()
