# -*- coding: utf-8 -*-
"""
Created on Fri Oct 18 11:43:09 2013

@author: root
"""
#from matplotlib import pyplot as plt
import numpy as np
import scipy.spatial.distance as sdist
def findCriticalPoints(inputimg, sigma):
# -*- coding: utf-8 -*-
    """
    Created on Wed Sep 18 09:56:20 2013
    find the critical points for the specified image and scale
    findCriticalPoints(inputimg, sigma)
    returns maxima, minima and saddle points
    """
    
#%% find the gaussian kernel for image    

    h2,h1 = inputimg.shape    
    x, y = np.mgrid[0:h2, 0:h1]
    x = x-h2/2
    y = y-h1/2
    #sigma = 1.5
    g = np.exp( -( x**2 + y**2 ) / (2*sigma**2) );
    gau =  g / g.sum()

#%% convolution of image with gaussian kernel in frequency domain

    Imgfft = np.fft.rfft2(inputimg)
    gfft = np.fft.rfft2(gau)
    fftimage = np.multiply(Imgfft, gfft)
    img =np.real(np.fft.ifftshift( np.fft.irfft2(fftimage))) * sigma

#%% shifting the matrix

    left = np.roll(img,-1,1) #shift the matrix one element left
    right = np.roll(img,1,1) #shift the matrix one element right
    up = np.roll(img,-1,0)   #shift the matrix one element up
    down = np.roll(img,1,0)  #shift the matrix one element down

    leftup = np.roll(left,-1,0)     #shift the matrix one element left and up
    leftdown = np.roll(left,1,0)    #shift the matrix one element left and down
    rightup = np.roll(right,-1,0)   #shift the matrix one element right and up
    rightdown = np.roll(right,1,0)  #shift the matrix one element right and down

#%%find the sign of the difference with center pixel

    row, col = img.shape
    signmat = np.zeros((row,col,8))
    signmat[:,:,0] = np.sign(img -right)
    signmat[:,:,1] = np.sign(img -rightup) #even
    signmat[:,:,2] = np.sign(img -up)
    signmat[:,:,3] = np.sign(img -leftup) #odd
    signmat[:,:,4] = np.sign(img -left)
    signmat[:,:,5] = np.sign(img -leftdown) #odd
    signmat[:,:,6] = np.sign(img -down)
    signmat[:,:,7] = np.sign(img -rightdown) #even

#odd find the total sign changes
    
    signodd = np.zeros((row,col,6))
    signodd[:,:,0] = signmat[:,:,0] + signmat[:,:,1] == 0
    signodd[:,:,1] = signmat[:,:,1] + signmat[:,:,2] == 0
    signodd[:,:,2] = signmat[:,:,2] + signmat[:,:,4] == 0
    signodd[:,:,3] = signmat[:,:,4] + signmat[:,:,6] == 0
    signodd[:,:,4] = signmat[:,:,6] + signmat[:,:,7] == 0
    signodd[:,:,5] = signmat[:,:,7] + signmat[:,:,0] == 0

#even find the total sign changes

    signeven = np.zeros((row,col,6))
    signeven[:,:,0] = signmat[:,:,0] + signmat[:,:,2] == 0
    signeven[:,:,1] = signmat[:,:,2] + signmat[:,:,3] == 0
    signeven[:,:,2] = signmat[:,:,3] + signmat[:,:,4] == 0
    signeven[:,:,3] = signmat[:,:,4] + signmat[:,:,5] == 0
    signeven[:,:,4] = signmat[:,:,5] + signmat[:,:,6] == 0
    signeven[:,:,5] = signmat[:,:,6] + signmat[:,:,0] == 0

#sum the 
    accumodd = np.sum(signodd,2)
    accumeven = np.sum(signeven,2)
    accumodd[range(0,row,2),:] = 0
    accumeven[range(1,row,2),:] = 0

    together = accumodd + accumeven
    together[0,:] = 2
    together[row-1,:] = 2
    together[:,0] = 2
    together[:,col-1] = 2

#    saddlepts = together == 4
    saddlepts = np.argwhere(together == 4)
#    saddlepts = np.float64(saddlepts)
    extremas = np.zeros((row,col,2))
    extremas[:,:,1] = together == 0

    extremasign = signmat[:,:,0]*extremas[:,:,1]
    minimapts = np.argwhere(extremasign == -1)    
#    minimapts = extremasign == -1
#    minimapts = np.float64(minimapts)    
    maximapts = np.argwhere(extremasign == 1)    
#    maximapts = extremasign == 1
#    maximapts = np.float64(maximapts)
    
    Iy, Ix=np.gradient(img)
    Ixy, Ixx = np.gradient(Ix)
    Iyy, Iyx = np.gradient(Iy)
#    dethessian = np.abs(Ixx + Iyy) #trace
    dethessian = np.abs(Ixx * Iyy - Ixy * Iyx) #determinant
#    return(dethessian)
    maxval = []
    for ii in range(maximapts.shape[0]):
        maxval.append((dethessian[maximapts[ii,0],maximapts[ii,1]]))

    maxval = np.array(maxval,'float32')
    
    return(maximapts, minimapts, saddlepts, maxval)



def trackCriticalPoints(maxpts):
    """
    created on 19 Oct 2013. function to track critical points and return the determinant of 
    hessian maximized scale space value of the critical path usage
    maxcrtipoint = trackCriticalPoints(maxpts)
    """
    maxptslabel = np.zeros((maxpts.shape[0],5))   
    maxptslabel[:, 0:4] = maxpts 
# initialize the labels for maxima points start by adding 100 to all the scale 
# zero points then find the euclidean distance to the point nearest to it above the scale
#     
    for jj in range(np.unique(maxpts[:,2]).shape[0] - 1):
        if (jj == 0):
            maxptslabel[maxptslabel[:, 2] == 0, 4] = np.add(100, range(maxptslabel[maxptslabel[:,2]==0,4].shape[0]))
        lower = maxptslabel[maxpts[:, 2] == jj]
        upper = maxptslabel[maxpts[:, 2] == jj + 1]
        distmat = sdist.cdist(lower[:, 0:2], upper[:,0:2] , 'euclidean')     
# assign the nearest point label to the upper scale        
        for ii in range(upper.shape[0]):
            upper[ii, 4] = lower[np.argmin(distmat[:, ii]), 4]
        #refinement step to eliminate more than one point in the higher scale being assigned same label
        for ii in range(upper.shape[0]):
            if (np.sum(upper[:,4] == upper[ii,4])>1 and upper[ii,4] != 0) :
                conflictlabel = upper[ii, 4]
                conflictarr = np.argwhere(upper[:, 4] == upper[ii,4])
                upper[conflictarr, 4 ] = 0
                upper[conflictarr[np.argmin(upper[conflictarr,3])], 4] = conflictlabel
        maxptslabel[maxpts[:, 2] == jj] = lower
        maxptslabel[maxpts[:, 2] == jj + 1] = upper
    tempmax = np.zeros((0,5))
    finmax = np.zeros((0,5))   
    finmaxlabel = np.unique(maxptslabel[:, 4])
    for ii in range(finmaxlabel.shape[0]):
        if(finmaxlabel[ii] != 0):
            maxresponse = np.argmax(maxptslabel[maxptslabel[:,4] == finmaxlabel[ii],3])
            condtn1 = maxptslabel[:, 2] == maxresponse 
            condtn2 = maxptslabel[:, 4] == finmaxlabel[ii]
            tempmax = maxptslabel[condtn1 * condtn2]
        finmax = np.concatenate((finmax,tempmax),0)
    return (finmax)
##    
    
def myshow(img):
    
    def onClick(event):
        print(img[event.ydata,event.xdata])     
        #    plt.close('all')
    plt.figure()    
    plt.ion() 
    plt.imshow(img,cmap='gray'),plt.show()
    fig = plt.gcf()
    #    on mouse click the value at the image location is displayed in output screen
    _ = fig.canvas.mpl_connect('button_press_event', onClick)


#img = plt.imread('n0.tif')
#cropimage = img[750:800,250:300]
##cropimage = img
#row, col = cropimage.shape 
#det = findCriticalPoints(cropimage, 3.)
#myshow(det)
#def plotCriticalPoints(cropimage, start, stop):
#    from matplotlib import pyplot as plt
#    import numpy as np
#    import criticalpoints
#    import time
#    starttime = time.time()
#    row, col = cropimage.shape 
#    maxpts = minpts = saddlepts = np.zeros((0,3))
#    scale = np.arange(start,stop,(stop-start)/18.0)
#    for count in range(18):    
#        maxima, minima, saddle = criticalpoints.findCriticalPoints(cropimage,scale[count])
#        maximascale = np.zeros((maxima.shape[0],3))
#        maximascale[:,0:2] = maxima
#        maximascale[:,2] = count
#        maxpts = np.concatenate((maxpts,maximascale),0)
#        
#        minimascale = np.zeros((minima.shape[0],3))
#        minimascale[:,0:2] = minima
#        minimascale[:,2] = count
#        minpts = np.concatenate((minpts,minimascale),0)
#        
#        saddlescale = np.zeros((saddle.shape[0],3))
#        saddlescale[:,0:2] = saddle
#        saddlescale[:,2] = count
#        saddlepts = np.concatenate((saddlepts,saddlescale),0)
#    
#    maxcolor = np.chararray((maxpts.shape[0],1))
#    maxcolor[:] = 'r'
#    mincolor = np.chararray((minpts.shape[0],1))
#    mincolor[:] = 'b'
#    saddlecolor = np.chararray((saddlepts.shape[0],1))
#    saddlecolor[:] = 'g'
#    allpts = np.concatenate((maxpts,minpts,saddlepts),0)
#    allcolor = np.concatenate((maxcolor,mincolor,saddlecolor),0)
#
#    from mpl_toolkits.mplot3d import axes3d
#    from matplotlib import cm
#    fig = plt.figure()
#    ax = fig.gca(projection='3d')
#    row,col = cropimage.shape
#    imgx, imgy = np.meshgrid(range(row),range(col))
#    plt.ion()    
#    ax.scatter(allpts[:,0], allpts[:,1], allpts[:,2],'z',30,allcolor)
#    plt.show()
#    return(maxpts, minpts, saddlepts)

#def findindex(array,value):
#    inc=0
#    for count in range(array.shape[0]):
#        if((array[count,:])==(value)):
#            index[inc]=count
#            inc=inc+1
#    return(index)
        

#def drawCircle(image,origin,radius,intensity):
#    #drawCircle(image,origin,radius,intensity)
#    import math
#    for ii in range((origin[0]-radius), (origin[1]+radius)):
#        for jj in range(origin[0]-radius, origin[1]+radius):
#            if(math.sqrt((origin[0]-ii)**2+(origin[1]-jj)**2) <= radius):
#                image[ii,jj] = intensity
#    return(image)
