# Project: pipeline v0.1
# Author: Praneeth Karempudi
# Email: praneeth.karempudi@gmail.com

#######################################################################
import torch
import torch.nn as nn
import torch.nn.functional as F
from .basicUnetParts import *

class Unetmod(nn.Module):
    '''
    Basic Unet stuff, simply copied the architecture with
    changes to sizes and layers, this one wont get 
    inputs from the side

    I am guessing they act a gradient highways preventing
    the net from learning features
    
    '''

    def __init__(self, model_expansion):
        # initialize the superclass initializer from the library modules
        super(Unetmod, self).__init__()
        # 1 because of the number of input channels is 1
        self.initial = double_conv(1, 8)
        self.down1 = double_conv(8, 16)
        self.down2 = double_conv(16, 32)
        self.down3 = double_conv(32, 64)
        self.down4 = double_conv(64, 64)

        self.up1 = up_conv(128, 32, model_expansion)
        self.up2 = up_conv(64, 16, model_expansion)
        self.up3 = up_conv(32, 8, model_expansion)
        self.up4 = up_conv(16, 8, model_expansion)
        self.out = nn.Conv2d(8, 1, 1) # 2 because of no_classes

    def forward(self, x):
        # x will be the image batch tensor, that will be propagated across
        # toward the end and the 
        x1 = self.initial(x)
        x2 = self.down1(F.max_pool2d(x1, (2, 2)))
        x3 = self.down2(F.max_pool2d(x2, (2, 2)))
        x4 = self.down3(F.max_pool2d(x3, (2, 2)))
        x5 = self.down4(F.max_pool2d(x4, (2, 2)))

        # copied code
        x = self.up1(x5, x4)
        x = self.up2(x, x3)
        x = self.up3(x, x2)
        x = self.up4(x, x1)
        x = self.out(x)


        return torch.sigmoid(x)
