# Project: pipeline v0.1
# Author: Praneeth Karempudi
# Email: praneeth.karempudi@gmail.com

##############################################################################
# This is the basic U net architecture, almost straight from the paper
# titled "U-Net: Convolutional Networks for Biomedical Image segmentation"
# by Olaf Ronneberger, Philipp Fischer and Thomas Brox, 2015
# PyTorch impelementation

import torch
import torch.nn as nn
import torch.nn.functional as F
from .basicUnetParts import *

class basicUnet(nn.Module):
    '''
    Basic Unet stuff, simply copied the architecture with
    changes to sizes and layers

    
    '''

    def __init__(self, model_expansion):
        # initialize the superclass initializer from the library modules
        super(basicUnet, self).__init__()
        # 1 because of the number of input channels is 1
        self.initial = double_conv(1, 64)
        self.down1 = double_conv(64, 128)
        self.down2 = double_conv(128, 256)
        self.down3 = double_conv(256, 512)
        self.down4 = double_conv(512, 512)

        self.up1 = up_conv(1024, 256, model_expansion)
        self.up2 = up_conv(512, 128, model_expansion)
        self.up3 = up_conv(256, 64, model_expansion)
        self.up4 = up_conv(128, 64, model_expansion)
        self.out = nn.Conv2d(64, 1, 1) # 2 because of no_classes

    def forward(self, x):
        # x will be the image batch tensor, that will be propagated across
        # toward the end and the 
        x1 = self.initial(x)
        x2 = self.down1(F.max_pool2d(x1, (2, 2)))
        x3 = self.down2(F.max_pool2d(x2, (2, 2)))
        x4 = self.down3(F.max_pool2d(x3, (2, 2)))
        x5 = self.down4(F.max_pool2d(x4, (2, 2)))

        # copied code
        x = self.up1(x5, x4)
        x = self.up2(x, x3)
        x = self.up3(x, x2)
        x = self.up4(x, x1)
        x = self.out(x)


        return torch.sigmoid(x)
