import numpy as np

#%###########################################################################
def gauss_kern(Img, sigmaVal=1.5):
    """ Returns a normalized 2D gauss kernel array for convolutions """
    h2,h1 = Img.shape    
    x, y = np.mgrid[0:h2, 0:h1]
    x = x-h2/2
    y = y-h1/2
    g = np.exp( -( x**2 + y**2 ) / (2*sigmaVal**2) );
    return g / g.sum()


#%###########################################################################
def enhanceContrast(orimg, sigmaVal):
    
    orimg = np.float32(orimg)
    #orimg = orimg.max()-orimg # uncomment for fluorescent data
    
    # find the eigen value of gaussian smoothed hessian image
    Img = np.copy(orimg)
    gau = gauss_kern(Img, sigmaVal)
    Imgfft = np.fft.rfft2(Img)
    gfft = np.fft.rfft2(gau)
    fftimage = np.multiply(Imgfft, gfft)
    Img_smooth =np.real(np.fft.ifftshift( np.fft.irfft2(fftimage)))
    Iy, Ix = np.gradient(Img_smooth)
    Ixy, Ixx = np.gradient(Ix)
    Iyy, Iyx = np.gradient(Iy)
    
    eigvallam = np.zeros((2,Ixx.shape[0],Ixx.shape[1] ))
    trhessian = Ixx+Iyy
    dethessian = Ixx*Iyy-Ixy*Ixy
    # find eigen values of hessian
    D = trhessian*trhessian - (4*dethessian)
    D[D<0] = 0
    eigvallam[0,:,:] = 0.5*(trhessian + np.sqrt(D))
    eigvallam[1,:,:] = 0.5*(trhessian - np.sqrt(D))
    eigh1 = eigvallam.min(0)
#    eigh2 = eigvallam.max(0)
    
    enhImage = np.copy(eigh1)
    enhImage = enhImage-enhImage.min()
    enhImage = enhImage/enhImage.max()
    enhImage = np.uint8(enhImage*255)

  #  enhImage2 = np.copy(eigh1)
   # if Img.shape[1] % 2 == 1:
    #    Img = Img[:,0:Img.shape[1]-1]
   # data = enhImage2[Img>0]
   # minVal = data.min()
   # maxVal = data.max()
  #  print "min1 :", minVal
   # print "min2 :", enhImage2.min()
    #enhImage2 = enhImage2-enhImage2.min()
    #print "max1 :", enhImage2[Img>0].max()
    #print "max2 :", enhImage2.max()
    #enhImage2 = enhImage2/enhImage2.max()
#    enhImage2 = (enhImage2-minVal)/maxVal
    #enhImage2 = np.uint8(enhImage2*255)
    #enhImage2[Img==0] = 0
    
    return(enhImage)#,enhImage2)

#%###########################################################################
def clearBorder(img,distThreshold):
    h2 = np.size(img,0)
    h1 = np.size(img,1)
    imBorder = np.concatenate((img[distThreshold-1], img[:,distThreshold-1],img[:,h1-distThreshold],img[h2-distThreshold]))
    for j in range(0,distThreshold-1):
        img[j] = 0
        img[:,j] = 0
        img[:,h1-1-j] = 0
        img[h2-1-j] = 0
    borderBlobs = np.unique(imBorder)
    borderBlobs = borderBlobs[borderBlobs>0]
    for blob in borderBlobs:
        img[img==blob] = 0

    return(img)