function [the_black,the_white,the_centroids] = detectRecolonization(channelData)
% Input is the pixels extracted from the growth channel center. Pixels are
% categorized into the three categories using k-means clustering algorithm:
% black, grey and white pixels. Clustering is done individually for each 
% growth channel
% 
% Output is the percent of black and white pixels in each frame for each 
% growth channel (trap).

channelLength = size(channelData,1);
nFrames = size(channelData,2);
nChannels = size(channelData,3);
the_black = zeros(nFrames,nChannels);
the_white = zeros(nFrames,nChannels);
the_centroids = zeros(nChannels,3);

for chIdx = 1:nChannels
    chIm = reshape(channelData(:,:,chIdx), [channelLength, nFrames]);        
    [segIm, k] = segmentImageWithClustering(chIm,3);

    the_centroids(chIdx,:) = k;
    the_black(:,chIdx) = sum(segIm==min(k))/channelLength*100;
    the_white(:,chIdx) = sum(segIm==max(k))/channelLength*100;
end