function [dotDistribution] = displayDotDistribution(analysisFolder, inBinX, inBinY)
%% Parameters
minCellLength = 10;
shift = 1;

%% Initialisation
posList = dir(strcat(analysisFolder,filesep,'pos*'));
nBinX = inBinX-shift;
nBinY = inBinY-shift;
dotDistribution = zeros(nBinY+shift,nBinX+shift);

%% Data extraction
for posIdx = 1:length(posList)
    currPos = posList(posIdx).name;
    
    listMat = dir(strcat(analysisFolder,filesep,currPos,filesep,'*',currPos,'.mat'));
    
    if isempty(listMat)
        continue
    end
    
    cellFile = load(strcat(analysisFolder,filesep,currPos,filesep,listMat(1).name));
    fieldNames = fieldnames(cellFile);
    cellsArray = cellFile.(fieldNames{1});
    
    disp(['Processing cells in ' currPos])
    for ii = length(cellsArray):-1:1
        if ~isempty(cellsArray(ii).cellLength)
            currCell = cellsArray(ii);
            if currCell.cellLength > minCellLength && currCell.cellWidth > 0
                for jj = 1:length(currCell.particles)
                    currPart = currCell.particles(jj);
                    for partT = 1:size(currPart.trajInternalCoord,3)
                        % Rescale the dot coord to the number of bins
                        currX = round(currCell.particles(jj).trajInternalCoord(partT,1)*nBinX+shift);
                        currY = round((1+currCell.particles(jj).trajInternalCoord(partT,2))*nBinY/2+shift);
                        
                        % Only accept dots with valid coordinates
                        if currX > 0 && currY > 0 && currX <= nBinX+shift && currY <= nBinY+shift
                            dotDistribution(currY,currX) = dotDistribution(currY,currX)+1;
                        end
                    end
                end
            end
        end
    end
end

figure, imagesc(dotDistribution), axis equal
