function [DCov,errorsCov,numberOfCells] = getCovarianceEstimator(outputFolder,timeStep,tE,pixelSize)
% This function computes apparent diffusion coefficients for trajectories
% of all sizes
%
% Input:
%   - outputFolder : folder that contains the results
%   - timeStep : time between frames [in ms]
%   - tE : exposure time [in ms]
%   - pixelSize : size of the pixels [in nm] (typical value is 80)
%
% Output:
%   - list of diffusion coefficients
%   - plot histogram for diffusion coefficients
%
% Author : Vladimir Curic, 2015-01-05

positionList = dir(strcat(outputFolder,filesep,'Pos*'));
DCov = [];
errorsCov = [];
numberOfCells = 0;

% bluring factor
R  = tE/(6*time*Step);

for positionIdx = 1:length(positionList)
    disp(positionList(positionIdx).name)
    
    cellMatList = dir(strcat(outputFolder,filesep,positionList(positionIdx).name,filesep,'*.mat'));
    cellMatFile = load(strcat(outputFolder,filesep,positionList(positionIdx).name,filesep,cellMatList(1).name));
    fieldName = fieldnames(cellMatFile);
    cellStruct = cellMatFile.(fieldName{1});
    
    numberOfCells = numberOfCells + size(cellStruct,2);
    
    for jj = 1:length(cellStruct)
        
        numberTempTrajectories = size(cellStruct(jj).particles,2);
        for kk = 1:numberTempTrajectories
            tempVar = cellStruct(jj).particles;
            tempTraj = tempVar(kk).trajImageCoord.*(pixelSize/1000);
            
            
            dX = diff(tempTraj);
            dX2 = dX.^2;
            dX2mean = mean(dX2);
            
            dXdX = mean(dX(1:end-1).*dX(2:end));
            
            % estimate D and sigma2
            DCov = [DCov,dX2mean/(2*timeStep)+dXdX/dt];
            errorsCov= [errorsCov,R*dX2mean+(2*R-1)*dXdX];
                     
        end
    end
end

% figure, hist(coeffDiff,50)
% %title('Diffusion coefficients')
% xlabel('Apparent diffusion coefficient [$\mu m^2 s^{-1}$]','Interpreter','latex')
% ylabel('Number of of trajectories','Interpreter','latex')

end



