function outCoord = convertCoord(coord, T, inverseFlag)
% outCoord = convertCoord(coord, T, inverseFlag)
% Converts coordinates coord according to the transformation defined by T.
%
% Input:
%   coord - matrix with each row representing dot information as [x y data]
%   T - 3x3 matrix, defines geometric trasformation
%   inverseFlag - logical, if 1, convert coordinates according to the 
%   inverse transformation w.r.t. T. Default is 0.
% 
% Output:
%   outCoord - matrix with transformed coordinates

if nargin == 2
    inverseFlag = 0;
end
if isempty(coord)
    outCoord = [];
    return;
end
inCoord(:,1:2) = double(coord(:,1:2));
inCoord(:,3) = 1;
if ~inverseFlag
    tempCoord = inCoord * T;
else
    tempCoord = inCoord / T;
end
outCoord = [tempCoord(:,1:2) coord(:,3:end)];