function [avgArea, sigma] = fitGaussianInitArea(areas, ax, pixelSize)
%areas,ax, doNormalization,pixelSize)
% [AVGAREA, SIGMA] = fitGaussianInitArea(AREAS)
% finds average value AVGAREA of single cell initiation areas in AREAS 
% by fitting a Gaussian with standard deviation SIGMA to the normalized
% data histogram. If R^2 coefficient < 0.7 or AREAS has less than 10
% values, NaN values are returned.
%
% fitGaussianInitArea(AREAS,DEBUG) plots histogram with fitted Gaussian if
% DEBUG=1. IF DEBUG is a handle to axes, plots into the axes.
% By default, DEBUG=1.
%
% fitGaussianInitArea(AREAS,DEBUG,PIXELSIZE) scales the data from px^2
% to um^2 using PIXELSIZE. By default, PIXELSIZE=1.
%
% Author: Spartak Zikrin, 2020-04-20

if nargin<2 || ax==1
    figure,
    ax=gca;
    pause(0.01);
end

if nargin < 3
    pixelSize = 1;
end

avgArea = nan;
sigma = nan;

if isempty(areas) || sum(~isnan(areas))<10
    return
end
areas = areas(~isnan(areas));
nBins = ceil((max(areas)-min(areas))/20);
[N, edges] = histcounts(areas,nBins);
x = edges(1:end-1) + (edges(2:end)-edges(1:end-1))/2;
N = N/sum(N);

avgArea = nan;
sigma = nan;
try
    [pdf, gof] = fit(x',N','gauss1');
catch ME
    throw(ME);
end

if gof.rsquare >= 0.7
    avgArea = pdf.b1;
    sigma = pdf.c1/sqrt(2);
end

avgArea = avgArea*pixelSize^2;

if ~isempty(ax) && isa(ax,'matlab.graphics.axis.Axes')
    x2 = x*pixelSize^2;
    step = (x2(end)-x2(1))/100;
    x3 = x2(1):step:x2(end);
    bar(ax,x2,N,1);
    hold(ax,'on'),
    plot(ax,x3,pdf(x3/pixelSize^2),'r-','Linewidth',2);
    plot(ax,[avgArea avgArea], [0 max(N)], 'r--','LineWidth',2);
    hold(ax,'off'),
end