function varargout = Visualizer(varargin)
% VISUALIZER MATLAB code for Visualizer.fig
%      VISUALIZER, by itself, creates a new VISUALIZER or raises the existing
%      singleton*. This is a visualization tool for ImAnalysis pipeline
%      that can plot cell images in preprocessed phase-contrast,
%      fluorescence, bright-field channels or segmentation masks. It can
%      also plot tracked cell boundaries, backbones and poles, 
%      detected dots and more.
%
%      H = VISUALIZER returns the handle to a new VISUALIZER or the handle to
%      the existing singleton*.
%
%      VISUALIZER('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in VISUALIZER.M with the given input arguments.
%
%      VISUALIZER(expInfoObj) creates a new VISUALIZER with data loaded
%      from expInfoObj.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Edit the above text to modify the response to help Visualizer

% Last modified: 2018-06-01, SZ. Fixed tag for disappearing cells.
% Last modified: 2018-03-23, SZ. Added segmented phase in the channels.

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @Visualizer_OpeningFcn, ...
                   'gui_OutputFcn',  @Visualizer_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before Visualizer is made visible.
function Visualizer_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to Visualizer (see VARARGIN)

% Choose default command line output for Visualizer
handles.output = hObject;
if length(varargin)>1
    errordlg('Visualizer accepts only a single input argument');
elseif length(varargin)==1
    if isa(varargin{1},'expInfo')
        handles = set_expinfo_object(handles, varargin{1});
    else
        errordlg('Input argument is not an expInfo object');
    end
end
% Update handles structure
guidata(hObject, handles);

% UIWAIT makes Visualizer wait for user response (see UIRESUME)
% uiwait(handles.figure1);


% --- Outputs from this function are returned to the command line.
function varargout = Visualizer_OutputFcn(hObject, eventdata, handles) 
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;


% --- Executes on button press in load_expinfo_button.
function load_expinfo_button_Callback(hObject, eventdata, handles)
% hObject    handle to load_expinfo_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
[filename, pathname, filterindex] = uigetfile();
if ~filterindex
    return
end
expInfoObj = loadExpInfo(pathname, filename);
handles = set_expinfo_object(handles, expInfoObj);
guidata(hObject, handles);



function handles = set_expinfo_object(handles, expInfoObj)
cla(handles.image_axes);
set(handles.image_axes,'Visible','off');
set(handles.image_filename_text,'String','');
handles.expInfoObj = expInfoObj;
set(handles.expinfo_file_text,'String','');
set(handles.position_menu,'String', [' ',handles.expInfoObj.getPositionList()],'Value',1);
set(handles.channel_menu,'String', [' ',handles.expInfoObj.getChannelNames(),'Seg'],'Value',1);

if handles.expInfoObj.getParameters('doBlobProcessing') ~= 2
    errordlg('Cell tracking is not done, nothing to display');
end
if handles.expInfoObj.getParameters('doDotDetection') ~= 2
    set(handles.dots_checkbox,'Value',0,'Enable','off');
else
    set(handles.dots_checkbox,'Value',0,'Enable','on');
end
if handles.expInfoObj.getParameters('doCellMeasurements') ~=2
    set(handles.backbone_checkbox,'Value',0,'Enable','off');
    set(handles.poles_checkbox,'Value',0,'Enable','off');
else
    set(handles.backbone_checkbox,'Value',0,'Enable','on');
    set(handles.poles_checkbox,'Value',0,'Enable','on');
end


% --- Executes on selection change in position_menu.
function position_menu_Callback(hObject, eventdata, handles)
% hObject    handle to position_menu (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = cellstr(get(hObject,'String')) returns position_menu contents as cell array
%        contents{get(hObject,'Value')} returns selected item from position_menu
posName = get_position_name(handles);
if strcmp(posName,' ')
    return
end

handles.frameList = set_frame_list(handles);

% read image directories for all channels
channelList = get(handles.channel_menu, 'String');
phaseChanName = handles.expInfoObj.getChannelNames('phase');
outputDir = handles.expInfoObj.getPathName('output');
sourceDir = handles.expInfoObj.getPathName('source');
for chIdx = 2:numel(channelList)
    chanName = channelList{chIdx};
    if strcmp(chanName,phaseChanName)
        imdirpath = fullfile(outputDir,posName,'PreprocessedPhase');
    elseif strcmp(chanName,'Seg')
        imdirpath = fullfile(outputDir,posName,'SegmentedPhase');
    else
        imdirpath = fullfile(sourceDir,posName,chanName);
    end
    handles.imdirPath{chIdx} = imdirpath;
    handles.imList{chIdx} = dir(fullfile(imdirpath,'*.tif*'));
    if ~strcmp(chanName,phaseChanName) && ~strcmp(chanName,'Seg')
        imRange = handles.expInfoObj.getRange(posName,chanName);
        if ~isempty(imRange)
            handles.imList{chIdx} = handles.imList{chIdx}(imRange);
        end
    end
end

% load MCell objects
loadpath = handles.expInfoObj.getMCellMatPath(posName);
handles.mCells = Cell.MCell.loadMCells(loadpath);
handles.mFrames = Cell.MCell.mFrames;

% Define cell colours
rng('default');
rng(1);
handles.cellColors = rand(length(handles.mCells),3);

handles.particles = getParticleList(handles.mCells);
cla(handles.image_axes);
handles = show_image(handles);
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function position_menu_CreateFcn(hObject, eventdata, handles)
% hObject    handle to position_menu (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on selection change in channel_menu.
function channel_menu_Callback(hObject, eventdata, handles)
% hObject    handle to channel_menu (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
if handles.expInfoObj.getParameters('doDotDetection') == 2    
    chanName = get_channel_name(handles);
    bfChanNames = handles.expInfoObj.getChannelNames('bf');
    if ismember(chanName,bfChanNames)
        set(handles.dots_checkbox,'Enable','off');
        set(handles.dots_checkbox,'Value',0);
    else
        set(handles.dots_checkbox,'Enable','on');
    end
end
handles.frameList = set_frame_list(handles);
cla(handles.image_axes);
handles = show_image(handles);
guidata(hObject,handles);

function position_name = get_position_name(handles)
positionList = cellstr(get(handles.position_menu, 'String'));
posIdx = get(handles.position_menu, 'Value');
position_name = positionList{posIdx};

function channel_name = get_channel_name(handles)
channelList = cellstr(get(handles.channel_menu, 'String'));
chanIdx = get(handles.channel_menu, 'Value');
channel_name = channelList{chanIdx};

function frameList = set_frame_list(handles)
posName = get_position_name(handles);
chanName = get_channel_name(handles);
if strcmp(chanName,'Seg')
    chanName = handles.expInfoObj.getChannelNames('phase');
end
if strcmp(posName,' ') || strcmp(chanName,' ')
    frameList.Indices = [];
    frameList.Range = [];
    
else
    frameList.Indices = handles.expInfoObj.getIndices(posName,chanName);
    frameList.Range = handles.expInfoObj.getRange(posName,chanName);
end
if isempty(frameList.Indices)
    set(handles.frame_list_index_edit,'String','');
else
    frameIdx = str2double(get(handles.frame_list_index_edit,'String'));
    frameListIdx  = getFrameListIndex(frameList.Indices,frameIdx);
    frameIdx = frameList.Indices(frameListIdx);
    set(handles.frame_list_index_edit,'String',frameIdx);
end

function frameListIdx = getFrameListIndex(frameIndices,frameIndex)
if isnan(frameIndex)
    frameListIdx = 1;
else
    frameListIdx = find(frameIndices>=frameIndex,1);
end


% --- Executes during object creation, after setting all properties.
function channel_menu_CreateFcn(hObject, eventdata, handles)
% hObject    handle to channel_menu (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% % --- Executes on button press in show_button.
% function show_button_Callback(hObject, eventdata, handles)
% % hObject    handle to show_button (see GCBO)
% % eventdata  reserved - to be defined in a future version of MATLAB
% % handles    structure with handles and user data (see GUIDATA)
% handles = show_image(handles);
% guidata(hObject,handles);

function handles = show_image(handles)
posName = get_position_name(handles);
chanName = get_channel_name(handles);
if strcmp(chanName,' ') || strcmp(posName,' ')
    return
end
frameIdx = str2double(get(handles.frame_list_index_edit,'String'));
if isempty(frameIdx) || isnan(frameIdx)
    errordlg('Wrong frame index');
end
phaseChanName = handles.expInfoObj.getChannelNames('phase');
phaseFrames = handles.expInfoObj.getIndices(posName,phaseChanName);
frameListIdx = getFrameListIndex(handles.frameList.Indices,frameIdx);
chIdx = get(handles.channel_menu,'Value');
filename = fullfile(handles.imdirPath{chIdx},handles.imList{chIdx}(frameListIdx).name);
set(handles.image_filename_text,'String',filename);
im = tiffread(filename);
if isempty(handles.mCells(1).boundaries)
    if strcmp(chanName,'Seg')
        segIm = im;
    else
        segListIdx = find(phaseFrames == frameIdx,1);
        segIm = imread(fullfile(handles.imdirPath{end},handles.imList{end}(segListIdx).name));
    end
    [B,L] = bwboundaries(segIm>0,'noholes');
end
if strcmp(chanName,'Seg')
    im = im>0;
end
if ~isempty(handles.image_axes.Children)
    xlimits = xlim(handles.image_axes);
    ylimits = ylim(handles.image_axes);
else
    xlimits = [];
end
imagesc(handles.image_axes,im), 
axis(handles.image_axes,'image'), 
colormap(handles.image_axes,'gray')
set(handles.image_axes,'Ydir','reverse','YTick',[],'XTick',[],'Clipping','off','Visible','off');
if ~isempty(xlimits)
    xlim(handles.image_axes,xlimits);
    ylim(handles.image_axes,ylimits);
end
min_clim = str2double(get(handles.min_clim,'String'));
max_clim = str2double(get(handles.max_clim,'String'));
if ~isempty(min_clim) && ~isempty(max_clim)
    the_clim = quantile(double(im(:)),[min_clim max_clim]);
    set(handles.image_axes,'Clim',the_clim);
end

doneMeasurements = strcmp(get(handles.backbone_checkbox,'Enable'),'on');

if strcmp(chanName, phaseChanName) || strcmp(chanName,'Seg')
    isPhaseChannel = true;
    phaseListIdx = frameListIdx;
else
    isPhaseChannel = false;
    phaseIndices = handles.expInfoObj.getIndices(posName,phaseChanName);
    T = handles.expInfoObj.getTByFrame(posName, frameListIdx, chanName);
    invT = inv(T);
    phaseListIdx = find(phaseIndices == frameIdx,1);
    fluoROI = [1 1 size(im,2) size(im,1)];
%     blobsInFrame = Spot.Refine.filterCellsByFluoRoi(handles.mFrames,phaseListIdx,frameROI,T);
end
blobsInFrame = handles.mFrames.getBlobsInFrame(phaseListIdx);
cellPoles = [];
BB = cell(size(blobsInFrame));
backBones = cell(size(BB));
okCell = true(size(BB));
cellEvents = cell(size(BB));
cellInfo = nan(length(BB),3);
counter = 0;
for blobLabel = blobsInFrame
    counter = counter+1;
    cellsInBlob = handles.mFrames.getCellsInBlob(phaseListIdx,blobLabel);
    c = cellsInBlob(1);
    t = c.getIndex(phaseListIdx);
    % get cell outline
    if ~isempty(c.boundaries)
        theBoundary = c.convertFromCellCoord(t,c.boundaries{t});
    else
        origo = round(c.centroids(t,:));
        label = L(origo(2),origo(1));
        if ~label
            idx = find(segIm==c.blobLabels(t),1);
            label = L(idx);
        end
        try
            theBoundary = B{label}(:,[2 1]);
        catch
            rr=1;
        end
    end
    if ~isPhaseChannel
        theBoundary = convertCoord(theBoundary, invT);
        if min(theBoundary(:, 1)) < fluoROI(1) || ...
           min(theBoundary(:, 2)) < fluoROI(2) || ...
           max(theBoundary(:, 1)) > fluoROI(1)+fluoROI(3)-1 || ...
           max(theBoundary(:, 2)) > fluoROI(2)+fluoROI(4)-1
            continue
        end
    end
    BB{counter} = theBoundary;%(:,[2 1]);
    cellInfo(counter,:) = [c.centroids(t,:) c.id];
    % define cell tag 
    if c.badSegmentations(t)
        okCell(counter) = false;
    end
    if frameListIdx>1
        if t==1
            if isempty(c.parent)
                cellEvents{counter} = 'appear';
            else
                cellEvents{counter} = 'born';
            end
        elseif t==c.lifeTime
            if isempty(c.descendants)
                cellEvents{counter} = 'disappear';
            else
                cellEvents{counter} = 'divide';
            end
        end
    end
    if doneMeasurements && c.lengths(t) > 0
        % compute cell backbone coordinates
        coeff = c.backbones(:,t);
        polesCellCoord = c.convertToCellCoord(t,c.poles{t},1);
        if ~c.isReoriented(t)
            step = 0.2;
        else
            step = -0.2;
        end
        bb = (polesCellCoord(1,1):step:polesCellCoord(2,1))';
        bb(:,2) = coeff(1)*bb(:,1).^2 + coeff(2)*bb(:,1) + coeff(3);
        bb = c.convertFromCellCoord(t, bb, 1);
        backBones{counter} = bb;
        % store cell poles
        cellPoles = [cellPoles; c.poles{t}];
    end
end
if ~isPhaseChannel
    ind = cellfun('isempty',BB);
%     BB = cellfun(@(x)convertCoord(x,invT,0),BB(~ind),'UniformOutput',false);
    okCell = okCell(~ind);
    cellEvents = cellEvents(~ind);
    ind = cellfun('isempty',backBones);
    backBones = cellfun(@(x)convertCoord(x,invT,0),backBones(~ind),'UniformOutput',false);
    cellPoles = convertCoord(cellPoles, invT);
    cellInfo = convertCoord(cellInfo, invT);
end
handles.cellBoundaries = cellfun(@(x)x(:,[2 1]),BB,'UniformOutput',false);
handles.okCell = okCell;
handles.cellEvents = cellEvents;
handles.backBones = backBones(~cellfun('isempty',backBones));
handles.cellPoles = cellPoles;
handles.centroids = round(cellInfo);

handles = plot_cell_outlines(handles);
handles = plot_poles(handles);
handles = plot_backbones(handles);
handles = plot_dots(handles);

function handles = plot_cell_outlines(handles)
if ~get(handles.cell_outlines_checkbox,'Value')
    return
end
BB = handles.cellBoundaries;
okCell = handles.okCell;
cellEvents = handles.cellEvents;
event_tags = {'appear','disappear','born','divide'};
    hold(handles.image_axes,'on');
hold(handles.image_axes,'on');
if get(handles.uniquecolor_checkbox,'Value')
    colorMap = handles.cellColors(handles.centroids(:,3),:);
    line_tags = repmat({'cell_outline'},size(BB));
    for t=1:length(event_tags)
        ind = strcmp(cellEvents,event_tags{t});
        line_tags(ind) = strcat(line_tags(ind),'_',event_tags{t});
    end
    for i=1:length(BB)
        p = line(BB{i}(:,2),BB{i}(:,1),'Color', colorMap(i,:));
        set(p,'Tag',line_tags{i});
    end
else
    p = visboundaries(BB(okCell & cellfun('isempty',cellEvents)),'Color','g','LineWidth',1,'EnhanceVisibility',false);
    set(p,'Tag','cell_outline_good');
    for i=1:length(event_tags)
        p = visboundaries(BB(okCell & strcmp(cellEvents,event_tags{i})),'Color','g','LineWidth',1,'EnhanceVisibility',false);
        set(p,'Tag',['cell_outline_good_' event_tags{i}]);
    end
    p = visboundaries(BB(~okCell & cellfun('isempty',cellEvents)),'Color','r','LineWidth',1,'EnhanceVisibility',false);
    set(p,'Tag','cell_outline_bad');
    for i=1:length(event_tags)
        p = visboundaries(BB(~okCell & strcmp(cellEvents,event_tags{i})),'Color','r','LineWidth',1,'EnhanceVisibility',false);
        set(p,'Tag',['cell_outline_bad_' event_tags{i}]);
    end
end
cells_to_show = get(handles.show_cells_buttongroup,'UserData');
if ~isempty(cells_to_show)
    objs = findobj(handles.image_axes,'-regexp','Tag','cell_outlines*');
    ind = ~contains({objs.Tag},cells_to_show);
    set(objs(ind),'Visible','off');
end
    
hold(handles.image_axes,'off');


function handles = plot_poles(handles)
poles = handles.cellPoles;
if strcmp(get(handles.poles_checkbox,'Enable'),'on') && ...
        get(handles.poles_checkbox','Value') && ...
        ~isempty(poles)
    hold(handles.image_axes,'on');
    p = line(handles.image_axes,poles(1:2:end,1),poles(1:2:end,2),...
        'Color','g','LineStyle','none','Marker','*');
    set(p,'Tag','cell_poles');
    hold(handles.image_axes,'off');
end

function handles = plot_backbones(handles)
bb = handles.backBones;
if get(handles.backbone_checkbox,'Value')
    hold(handles.image_axes,'on');
    for i=1:length(bb)
        p = line(bb{i}(:,1),bb{i}(:,2),'Color','y');
        set(p,'Tag','cell_backbone');
    end
    hold(handles.image_axes,'off');
end

function handles = plot_dots(handles)
if get(handles.dots_checkbox,'Value')
    phaseFrame = str2double(get(handles.frame_list_index_edit,'String'));
    chanName = get_channel_name(handles);
    phaseChanName = handles.expInfoObj.getChannelNames('phase');
    fluoChanNames = handles.expInfoObj.getChannelNames('fluo');
    posName = get_position_name(handles);
    phaseFrames = handles.expInfoObj.getIndices(posName,phaseChanName);
    detectionFrame = find(phaseFrames==phaseFrame);
    if strcmp(chanName, phaseChanName) || strcmp(chanName,'Seg')
        styles = {'r.','c.','m.'};
        dotInd = handles.particles.detectionFrame == detectionFrame;
        if any(dotInd)
            hold(handles.image_axes,'on');
            for i=1:numel(fluoChanNames)
                fluoChanName = fluoChanNames{i};
                ind = dotInd & strcmp(handles.particles.chanName,fluoChanName);
                coord = handles.particles.phaseCoord(ind,:);
%                 radii = 3*ones(size(coord,1),1);
%                 p = viscircles(handles.image_axes,coord,radii,'Color',dotColors{i},'LineWidth',1,'EnhanceVisibility',false);
%                 set(p,'Tag','particles');
                p = plot(handles.image_axes,coord(:,1),coord(:,2),styles{i});
                set(p,'Tag','particles');
            end
            hold(handles.image_axes,'off');
            dcm = datacursormode(handles.figure1);
            set(dcm, 'UpdateFcn', {@datatip_callback, ...
                handles.particles.phaseCoord(dotInd,:), ...
                handles.particles.internalCoord(dotInd,:),...
                handles.centroids});
        end
    elseif ismember(chanName, fluoChanNames)
        dotInd = handles.particles.detectionFrame == detectionFrame & ...
                 strcmp(handles.particles.chanName,chanName);
        if any(dotInd)
            hold(handles.image_axes,'on');
            coord = handles.particles.fluoCoord(dotInd,:);
%             radii = 3*ones(size(coord,1),1);
%             p = viscircles(handles.image_axes,coord,radii,'Color','r','LineWidth',1,'EnhanceVisibility',false);
%             set(p,'Tag','particles');
            p = line(handles.image_axes,coord(:,1),coord(:,2),'Color','r','LineStyle','none','Marker','.');
            set(p,'Tag','particles');
            hold(handles.image_axes,'off');
            dcm = datacursormode(handles.figure1);
            set(dcm, 'UpdateFcn', {@datatip_callback, ...
                handles.particles.fluoCoord(dotInd,:), ...
                handles.particles.internalCoord(dotInd,:),...
                handles.centroids});
        end
    end
else
    dcm = datacursormode(handles.figure1);
    set(dcm, 'UpdateFcn', {@datatip_callback, [], [], handles.centroids});
end

function outtxt = datatip_callback(obj, eventObj, origCoord,internalCoord,cellInfo)

coord = get(eventObj,'Position');

if isempty(origCoord) || isempty(internalCoord)
    idx = 0;
else
    idx = origCoord(:,1) == coord(1) & origCoord(:,2) == coord(2);
end
if any(idx)
    outtxt = sprintf('%f, %f', internalCoord(idx, 1),internalCoord(idx, 2));
else
    dist = sqrt((cellInfo(:,1)-coord(1)).^2+(cellInfo(:,2)-coord(2)).^2);
    idx = find(dist<3,1);
    if ~isempty(idx)
        outtxt = sprintf('cell %d', cellInfo(idx,3));
    else
        outtxt = sprintf('X:%d\nY:%d', coord(1),coord(2));
    end
end

% --- Executes on button press in cell_outlines_checkbox.
function cell_outlines_checkbox_Callback(hObject, eventdata, handles)
% hObject    handle to cell_outlines_checkbox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
if get(hObject,'Value')
    state = 'on';
else
    state = 'off';
end
objs = findobj(handles.image_axes,'-regexp','Tag','cell_outline*');
if ~isempty(objs)
    set(objs,'Visible',state);
else
    handles = plot_cell_outlines(handles);
end
guidata(hObject,handles);

% --- Executes on button press in dots_checkbox.
function dots_checkbox_Callback(hObject, eventdata, handles)
% hObject    handle to dots_checkbox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
if get(hObject,'Value')
    state = 'on';
else
    state = 'off';
end
objs = findobj(handles.image_axes,'Tag','particles');
if isempty(objs)
    handles = plot_dots(handles);
else
    set(objs,'Visible',state);
end
guidata(hObject,handles);


% --- Executes on button press in backbone_checkbox.
function backbone_checkbox_Callback(hObject, eventdata, handles)
% hObject    handle to backbone_checkbox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
if get(hObject,'Value') == 1
    state = 'on';
else
    state = 'off';
end
objs = findobj(handles.image_axes,'Tag','cell_backbone');
if isempty(objs)
    handles = plot_backbones(handles);
else
    set(objs,'Visible',state);
end
guidata(hObject,handles);


% --- Executes on button press in poles_checkbox.
function poles_checkbox_Callback(hObject, eventdata, handles)
% hObject    handle to poles_checkbox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
if get(hObject,'Value') == 1
    state = 'on';
else
    state = 'off';
end
h = findobj(handles.image_axes,'Tag','cell_poles');
if ~isempty(h)
    set(h,'Visible',state);
elseif strcmp(state,'on')
    handles = plot_poles(handles);
end
guidata(hObject,handles);



function min_clim_Callback(hObject, eventdata, handles)
% hObject    handle to min_clim (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
min_clim = str2double(get(hObject,'String'));
max_clim = str2double(get(handles.max_clim,'String'));
if min_clim<0 || min_clim>max_clim || max_clim>1
    errordlg('Color limits must be between 0 and 1');
end
im = get(findobj(handles.image_axes,'Type','Image'),'CData');
the_clim = quantile(double(im(:)),[min_clim max_clim]);
set(handles.image_axes,'Clim',the_clim);
guidata(hObject,handles);


% --- Executes during object creation, after setting all properties.
function min_clim_CreateFcn(hObject, eventdata, handles)
% hObject    handle to min_clim (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function max_clim_Callback(hObject, eventdata, handles)
% hObject    handle to max_clim (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
min_clim = str2double(get(handles.min_clim,'String'));
max_clim = str2double(get(hObject,'String'));
if min_clim<0 || min_clim>max_clim || max_clim>1
    errordlg('Color limits must be between 0 and 1');
end
im = get(findobj(handles.image_axes,'Type','Image'),'CData');
the_clim = quantile(double(im(:)),[min_clim max_clim]);
set(handles.image_axes,'Clim',the_clim);
guidata(hObject,handles);
% Hints: get(hObject,'String') returns contents of max_clim as text
%        str2double(get(hObject,'String')) returns contents of max_clim as a double


% --- Executes during object creation, after setting all properties.
function max_clim_CreateFcn(hObject, eventdata, handles)
% hObject    handle to max_clim (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in forward_button.
function forward_button_Callback(hObject, eventdata, handles)
% hObject    handle to forward_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
frameIdx = str2double(get(handles.frame_list_index_edit,'String'));
frameListIdx = getFrameListIndex(handles.frameList.Indices,frameIdx);
if isempty(frameListIdx) || isnan(frameListIdx)
    return;
end
step = str2double(get(handles.step_size,'String'));
if frameListIdx <= numel(handles.frameList.Indices)-step
    frameListIdx = frameListIdx+step;
else
    frameListIdx = 1;
end
nextFrameIdx = handles.frameList.Indices(frameListIdx);
set(handles.frame_list_index_edit,'String',nextFrameIdx);
handles = show_image(handles);
guidata(hObject,handles);

% --- Executes on button press in backward_button.
function backward_button_Callback(hObject, eventdata, handles)
% hObject    handle to backward_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
frameIdx = str2double(get(handles.frame_list_index_edit,'String'));
frameListIdx = getFrameListIndex(handles.frameList.Indices,frameIdx);
if isempty(frameListIdx) || isnan(frameListIdx)
    return;
end
step = str2double(get(handles.step_size,'String'));
if frameListIdx > step
    frameListIdx = frameListIdx - step;
else
    frameListIdx = numel(handles.frameList.Indices);
end
nextFrameIdx = handles.frameList.Indices(frameListIdx);
set(handles.frame_list_index_edit,'String',nextFrameIdx);
handles = show_image(handles);
guidata(hObject,handles);

function frame_list_index_edit_Callback(hObject, eventdata, handles)
% hObject    handle to frame_list_index_edit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of frame_list_index_edit as text
%        str2double(get(hObject,'String')) returns contents of frame_list_index_edit as a double
handles = show_image(handles);
guidata(hObject,handles);

% --- Executes during object creation, after setting all properties.
function frame_list_index_edit_CreateFcn(hObject, eventdata, handles)
% hObject    handle to frame_list_index_edit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function step_size_Callback(hObject, eventdata, handles)
% hObject    handle to step_size (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of step_size as text
%        str2double(get(hObject,'String')) returns contents of step_size as a double


% --- Executes during object creation, after setting all properties.
function step_size_CreateFcn(hObject, eventdata, handles)
% hObject    handle to step_size (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in uniquecolor_checkbox.
function uniquecolor_checkbox_Callback(hObject, eventdata, handles)
% hObject    handle to uniquecolor_checkbox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
objs = findobj(handles.image_axes,'-regexp','Tag','cell_outline_');
delete(objs);
handles = plot_cell_outlines(handles);
guidata(hObject,handles);


% --- Executes on button press in appear_radiobutton.
function appear_radiobutton_Callback(hObject, eventdata, handles)
% hObject    handle to appear_radiobutton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
if get(hObject,'Value')
    objs = findobj(handles.image_axes,'-regexp','Tag','cell_outline*');
    ind = contains({objs.Tag},'_appear');
    set(objs(ind),'Visible','on');
    set(objs(~ind),'Visible','off');
    set(handles.show_cells_buttongroup,'UserData','appear');
    guidata(hObject,handles);
end 


% --- Executes on button press in disappear_radiobutton.
function disappear_radiobutton_Callback(hObject, eventdata, handles)
% hObject    handle to disappear_radiobutton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
if get(hObject,'Value')
    objs = findobj(handles.image_axes,'-regexp','Tag','cell_outline*');
    ind = contains({objs.Tag},'_disappear');
    set(objs(ind),'Visible','on');
    set(objs(~ind),'Visible','off');
    set(handles.show_cells_buttongroup,'UserData','disappear');
    guidata(hObject,handles);
end 


% --- Executes on button press in divide_radiobutton.
function divide_radiobutton_Callback(hObject, eventdata, handles)
% hObject    handle to divide_radiobutton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
if get(hObject,'Value')
    objs = findobj(handles.image_axes,'-regexp','Tag','cell_outline*');
    ind = contains({objs.Tag},'_divide');
    set(objs(ind),'Visible','on');
    set(objs(~ind),'Visible','off');
    set(handles.show_cells_buttongroup,'UserData','divide');
    guidata(hObject,handles);
end 

% --- Executes on button press in born_radiobutton.
function born_radiobutton_Callback(hObject, eventdata, handles)
% hObject    handle to born_radiobutton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
if get(hObject,'Value')
    objs = findobj(handles.image_axes,'-regexp','Tag','cell_outline*');
    ind = contains({objs.Tag},'_born');
    set(objs(ind),'Visible','on');
    set(objs(~ind),'Visible','off');
    set(handles.show_cells_buttongroup,'UserData','born');
    guidata(hObject,handles);
end 

% --- Executes on button press in all_radiobutton.
function all_radiobutton_Callback(hObject, eventdata, handles)
% hObject    handle to all_radiobutton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
if get(hObject,'Value')
    objs = findobj(handles.image_axes,'-regexp','Tag','cell_outline*');
    set(objs,'Visible','on');
    set(handles.show_cells_buttongroup,'UserData','');
    guidata(hObject,handles);
end 


% --- Executes on button press in slideshow_button.
function slideshow_button_Callback(hObject, eventdata, handles)
% hObject    handle to slideshow_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
frameIdx = str2double(get(handles.frame_list_index_edit,'String'));
if isempty(frameIdx) || isnan(frameIdx)
    errordlg('Wrong frame index');
end
state = str2double(get(handles.pause_edit,'String'));
if isnan(state) || isempty(state)
    errordlg('Set correct slideshow pause duration, in seconds');
end
set(hObject,'UserData',1);
currFrameListIdx = getFrameListIndex(handles.frameList.Indices,frameIdx);
step = str2double(get(handles.step_size,'String'));
pause(state);
for frameListIdx = currFrameListIdx+step:step:numel(handles.frameList.Indices)
    if ~get(hObject,'UserData')
        break
    end
    frameIdx = handles.frameList.Indices(frameListIdx);
    set(handles.frame_list_index_edit,'String',frameIdx);
    handles = show_image(handles);
    pause(state);
    guidata(hObject,handles);
end


function pause_edit_Callback(hObject, eventdata, handles)
% hObject    handle to pause_edit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of pause_edit as text
%        str2double(get(hObject,'String')) returns contents of pause_edit as a double


% --- Executes during object creation, after setting all properties.
function pause_edit_CreateFcn(hObject, eventdata, handles)
% hObject    handle to pause_edit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in stop_slideshow_button.
function stop_slideshow_button_Callback(hObject, eventdata, handles)
% hObject    handle to stop_slideshow_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
set(handles.slideshow_button,'UserData',0);
guidata(hObject,handles);
