function convertMCellToACell(analysisDir, outputDir)
% This function is used to convert MCell objects to aCell objects which
% were used in earlier versions of the pipeline. It is used for HMM pipeline
% 
% Input:
%   analysisDir - path to directory with ImAnalysis output
%   outputDir - path to directory where to store aCell objects
%
% SZ, 2019-02-06

expInfoObj = loadExpInfo(analysisDir);
positionList = expInfoObj.getPositionList();
for posIdx = 1:length(positionList)
    posName = positionList{posIdx};
    mCellsMatFile = expInfoObj.getMCellMatPath(posName);
    mCells = Cell.MCell.loadMCells(mCellsMatFile);
    nCells = length(mCells);
    cellsArray(1,1:nCells) = FastSeg.aCell;
    for cIdx = 1:nCells
        c = mCells(cIdx);
        particles = c.particles;
        newParticles =  struct('channelName',       [], ...
                                     'trajImageCoord',    {}, ...
                                     'trajImageCov',      {}, ...
                                     'trajInternalCoord', {}, ...
                                     'numberOfFrame',     [], ...
                                     'firstFrame',        [], ...
                                     'fluoTimes',         [], ...
                                     'localisationError', [], ...
                                     'extraInfo',         [], ...
                                     'dotToDotDist',      []);
        for partIdx = 1:length(particles)
            newParticles(partIdx).trajImageCoord = particles(partIdx).trajImageCoord;
            newParticles(partIdx).trajImageCov = particles(partIdx).trajImageCov;
            newParticles(partIdx).trajInternalCoord = particles(partIdx).trajInternalCoord;
            newParticles(partIdx).numberOfFrame = particles(partIdx).numberOfFrame;
            newParticles(partIdx).firstFrame = particles(partIdx).firstFrame;
            newParticles(partIdx).fluoTimes = particles(partIdx).fluoTimes;
            newParticles(partIdx).localisationError = [];
            newParticles(partIdx).dotToDotDist = [];
        end
        isRotated = zeros(1,c.lifeTime);
        for t=1:c.lifeTime
            isRotated(t) = c.isRotated(t);
        end
        cellsArray(cIdx) = FastSeg.aCell(...
            'cellId',c.id,...
            'birthFrame', c.birthFrame, ...
            'parent', [], ...
            'descendants', [], ...
            'cellLabels', c.blobLabels, ...
            'fluoMovieIdx', [], ...
            'backBoneFit', c.backbones, ...
            'isRotated', isRotated, ...
            'cellLength', c.lengths, ...
            'cellWidth', c.widths, ...
            'majorAxisOrientationDegree', c.majorAxisOrientationDegree, ...
            'fluoToSegScaling', [1 1], ...
            'particles', newParticles);
    end
    save( fullfile(outputDir,sprintf('outCells_%s.mat',posName)), 'cellsArray');
    clear cellsArray
end