classdef readAsFrames
    properties (GetAccess=private)
        positions = {};
        positions_map = [];
        timepoints = [];
        unique_timepoints = {};
        acquire_times = {};
        channels = {};
        img_nr_adapter = {};
        exposuretimes = {};
        abs_path = '';
        prefix = 'img_';
        name_sep = '_';
        phase_ch_name = '';
        bright_ch_name = '';
        fluor_ch_name = '';
        pos_ch_counters = [];
        pos_frame_counters = [];

    end
    
    methods
        function img = getImg(obj, position_index, frame_index, img_type)
            img_nr = obj.img_nr_adapter{position_index}{frame_index}(img_type);
            
            switch img_type 
              case 1
                ch_name = obj.phase_ch_name;
              case 2
                ch_name = obj.bright_ch_name;
              case 3
                ch_name = obj.fluor_ch_name;
            end
            
            if (img_nr ~= -1 )
                filename = [obj.abs_path filesep obj.positions{position_index}...
                            filesep ch_name filesep obj.prefix...
                            obj.name_sep sprintf('%09d', img_nr)...
                            obj.name_sep...
                            num2str(obj.exposuretimes{position_index}{frame_index}(img_type))...
                            obj.name_sep sprintf('%03d', 0)...
                            '.tiff'];
                
                img = uint16(imread(filename));
            else
                img = -1;
            end
        end
        
        function indecies = getIndecies(obj, position_index, type_index)
            indecies = zeros(obj.pos_frame_counters(position_index, type_index),1);
            
            ind_length = length(indecies);
            
            counter = 0;
            for i = 1:length(obj.img_nr_adapter{position_index})
                img_nr = obj.img_nr_adapter{position_index}{i}(type_index);
                if (img_nr ~= -1 )
                    counter = counter + 1;
                    indecies(counter) = i;
                    
                end
            end
            
            if ind_length ~= counter
                disp('Error: Index mismatch!')
            end
        end

        function times = getAcquireTimes(obj, position_name, type_index)
            position_index = obj.positions_map(position_name);
            times = zeros(length(obj.acquire_times{position_index}),1);
            for i = 1:length(obj.acquire_times{position_index});
                times(i) = obj.acquire_times{position_index}{i}(type_index);
            end
        end
        function times = getUniqueTimes(obj, position_name)
            position_index = obj.positions_map(position_name);
            times = zeros(length(obj.unique_timepoints{position_index}),1);
            for i = 1:length(obj.unique_timepoints{position_index});
                times(i) = obj.unique_timepoints{position_index};
            end
        end
        
    end
    
    methods
        function obj = readAsFrames(metafile, working_dir,phase_ch_name,bright_ch_name,fluor_ch_name)
            import json.*;
            
            obj.abs_path = working_dir;
            metafile_str = fileread([working_dir filesep ...
                                metafile]);
            
            obj.positions = json.getPositionNames(metafile_str);
            obj.positions_map = containers.Map(obj.positions,...
                                               1:length(obj.positions));
            obj.channels  = json.getChannelNames(metafile_str);
            
            obj.phase_ch_name=phase_ch_name;
            obj.bright_ch_name=bright_ch_name;
            obj.fluor_ch_name=fluor_ch_name;

            obj.pos_ch_counters = zeros(length(obj.positions), 3);
            obj.pos_frame_counters = zeros(length(obj.positions), 3);
            
            for i = 1:length(obj.positions)
                obj.unique_timepoints{i} = ...
                    unique(json.getExpectedTimePoints(metafile_str, ...
                                               obj.positions{i}, '*'));
            end
            for i = 1:length(obj.positions)
                for j = 1:length(obj.unique_timepoints{i})
                    obj.img_nr_adapter{i}{j} = [-1 -1 -1];
                    obj.exposuretimes{i}{j} = [-1 -1 -1];
                    obj.acquire_times{i}{j} = [-1 -1 -1];
                end
            end
            
            all_timepoints = getJson(metafile_str);
            for i = 1:length(all_timepoints)
                cur_pos = all_timepoints{i};
                cur_expectedtime = ...
                    int64(str2double(cur_pos.expected_acquire_time));
                cur_pos_i = obj.positions_map(cur_pos.position);
                timepoint_i = ...
                    find(obj.unique_timepoints{cur_pos_i}...
                         == cur_expectedtime);
                
                
                if strcmp(cur_pos.channel, obj.phase_ch_name)
                    
                    % disregard all doublett shots of frames.
                    if obj.img_nr_adapter{cur_pos_i}{timepoint_i}(1) ...
                            == -1
                        obj.img_nr_adapter{cur_pos_i}{timepoint_i}(1) ...
                            = obj.pos_ch_counters(cur_pos_i,1);
                        
                        obj.exposuretimes{cur_pos_i}{timepoint_i}(1) ...
                            = str2num(cur_pos.exposure_time);
                        
                        obj.acquire_times{cur_pos_i}{timepoint_i}(1)...
                            = str2num(cur_pos.acquire_time);
                        
                        obj.pos_frame_counters(cur_pos_i,1) = ...
                            obj.pos_frame_counters(cur_pos_i,1) + 1;

                    end
                    obj.pos_ch_counters(cur_pos_i,1) = ...
                        obj.pos_ch_counters(cur_pos_i,1) + 1;
                    
                elseif strcmp(cur_pos.channel, obj.bright_ch_name)
                    
                    % disregard all doublett shots of frames.
                    if obj.img_nr_adapter{cur_pos_i}{timepoint_i}(2) ...
                            == -1
                        obj.img_nr_adapter{cur_pos_i}{timepoint_i}(2) ...
                            = obj.pos_ch_counters(cur_pos_i,2);
                        
                        obj.exposuretimes{cur_pos_i}{timepoint_i}(2) ...
                            = int32(str2double(cur_pos.exposure_time));
                        
                        obj.acquire_times{cur_pos_i}{timepoint_i}(2)...
                            = str2num(cur_pos.acquire_time);

                        obj.pos_frame_counters(cur_pos_i,2) = ...
                            obj.pos_frame_counters(cur_pos_i,2) + 1;

                    end

                    obj.pos_ch_counters(cur_pos_i,2) = ...
                        obj.pos_ch_counters(cur_pos_i,2) + 1;
                    
                elseif strcmp(cur_pos.channel, obj.fluor_ch_name)

                    % disregard all doublett shots of frames.
                    if obj.img_nr_adapter{cur_pos_i}{timepoint_i}(3) ...
                            == -1
                        obj.img_nr_adapter{cur_pos_i}{timepoint_i}(3) ...
                            = obj.pos_ch_counters(cur_pos_i,3);
                        
                        obj.exposuretimes{cur_pos_i}{timepoint_i}(3) ...
                            = int32(str2double(cur_pos.exposure_time));
                        
                        obj.acquire_times{cur_pos_i}{timepoint_i}(3)...
                            = str2num(cur_pos.acquire_time);

                        obj.pos_frame_counters(cur_pos_i,3) = ...
                            obj.pos_frame_counters(cur_pos_i,3) + 1;

                    end

                    obj.pos_ch_counters(cur_pos_i,3) = ...
                        obj.pos_ch_counters(cur_pos_i,3) + 1;
                    
                end
            end
        end
        
        
        function img = getPhaseImg(obj, position_index, frame_index)
            img = obj.getImg(position_index, frame_index, 1);
        end
        
        function img = getPhaseImgByName(obj, position_name, frame_index)
            position_index = obj.positions_map(position_name);
            img = obj.getPhaseImg(position_index, ...
                                        frame_index);
        end

        function img_nr = getPhaseImgNr(obj, position_index, frame_index)
            img_nr = obj.img_nr_adapter{position_index}{frame_index}(1);
        end
        
        function img = getBrightImg(obj, position_index, frame_index)
            img = obj.getImg(position_index, frame_index, 2);
        end

        function img = getBrightImgByName(obj, position_name, frame_index)
            position_index = obj.positions_map(position_name);
            img = obj.getBrightImg(position_index, frame_index);
        end

        function img = getFluorImg(obj, position_index, frame_index)
            img = obj.getImg(position_index, frame_index, 3);
        end
        
        function img = getFluorImgByName(obj, position_name, frame_index)
            position_index = obj.positions_map(position_name);
            img = obj.getFluorImg(position_index, frame_index);
        end

        
        function indecies = getPhaseIndecies(obj, position_index)
            indecies = getIndecies(obj, position_index, 1);
        end
        
        function indecies = getPhaseIndeciesByName(obj, position_name)
            position_index = obj.positions_map(position_name);
            indecies = obj.getPhaseIndecies(position_index);
        end
        
        
        function indecies = getBrightIndecies(obj, position_index)
            indecies = getIndecies(obj, position_index, 2);
        end
                
        function indecies = getBrightIndeciesByName(obj, position_name)
            position_index = obj.positions_map(position_name);
            indecies = obj.getBrightIndecies(position_index);
        end
        
        
        function indecies = getFluorIndecies(obj, position_index)
            indecies = getIndecies(obj, position_index, 3);
        end
                
        function indecies = getFluorIndeciesByName(obj, position_name)
            position_index = obj.positions_map(position_name);
            indecies = obj.getFluorIndecies(position_index);
        end
        
        
        function times = getPhaseAcquireTimes(obj, position_name)
            times = getAcquireTimes(obj, position_name, 1);
        end
        
        function times = getBrightAcquireTimes(obj, position_name)
            times = getAcquireTimes(obj, position_name, 2);
        end
        
        function times = getFluorAcquireTimes(obj, position_name)
            times = getAcquireTimes(obj, position_name, 3);
        end
        
        function times = getBrightUniqueTimes(obj, position_name)
            ind = getBrightIndeciesByName(obj, position_name);
            position_index = obj.positions_map(position_name);
            times=zeros(length(obj.unique_timepoints{position_index}),1);
            times(ind) = obj.unique_timepoints{position_index}(ind);
        end
        
        function times = getFluorUniqueTimes(obj, position_name)
            %             times = getUniqueTimes(obj, position_name, 3);
            ind = getFluorIndeciesByName(obj, position_name);
            position_index = obj.positions_map(position_name);
            times=zeros(length(obj.unique_timepoints{position_index}),1);
            times(ind) = obj.unique_timepoints{position_index}(ind);
        end
        
        function times = getPhaseUniqueTimes(obj, position_name)
%             times = getUniqueTimes(obj, position_name, 1);
            ind = getPhaseIndeciesByName(obj, position_name);
            position_index = obj.positions_map(position_name);
            times=zeros(length(obj.unique_timepoints{position_index}),1);
            times(ind) = obj.unique_timepoints{position_index}(ind);
        end
        
        function positions = getPositionList(obj)
            positions = obj.positions;
        end
    end
end
