from __future__ import print_function, division
import os
import torch
import re
from torch.utils.data import Dataset, DataLoader
from torchvision import transforms, utils
from supporting_functions.loadDataSets import mmDataSetSegment
import supporting_functions.customTransformations as ct
from supporting_functions.loadFiles import getFileList
from supporting_functions.mmSegmentation import mmSegmentation
import numpy as np
from skimage import io
import time

#import unets
from networks.PraNetUnet import PraNetUnet, ShallowPraNetUnet
from networks.UnetPlusPlus2 import NestedUNet, UNet, ShallowNestedUNet

phase_dir = '/hdd/RecPAIR/Microscopy/EXP-20-BQ2490 RecA-SYFP2/analysis20200424/exp3_5minInd_LED/prod/'
ims = sorted(getFileList(phase_dir, chan='PreprocessedPhase'))
# phase_dir = '/hdd/RecPAIR/Microscopy/Praneeth_ims/'
# ims = sorted(getFileList(phase_dir, chan='PreprocessedPhase'))
save_dir = 'SegmentedChannels'

#make directories to put cell in
for (root, dirs, files) in os.walk(phase_dir):
    if re.search('PreprocessedPhase',root) is not None:
        seg_dir = root.replace('PreprocessedPhase',save_dir)
        if not os.path.exists(seg_dir):
            os.makedirs(seg_dir)
        else:
            print('PreprocessedPhase directory already exists')
            break

seg_transformations = transforms.Compose([transforms.ToPILImage(),
                                          ct.PILPadTo16(),
                                          transforms.ToTensor(),
                                          ct.SegmentationNormalize()]) #add normalisatin transformation

ims_to_segment = mmDataSetSegment(phase_ims=ims,
                                  transform=seg_transformations, 
                                  output_folder=save_dir)

segmentation_loader = DataLoader(ims_to_segment, batch_size=1, shuffle=False, num_workers=16)


net = ShallowNestedUNet(deep_supervision=False)
# saved_model = '/hdd/RecPAIR/ShallowUnet_mixed_brightnessAdj_RMSprop_400px_cells.pth'
saved_model = '/hdd/RecPAIR/trained_unets/ShNestedUnet_mixed_brightnessAdj_Adam__HybridLoss_256px_channels.pth'
# saved_model =  '/hdd/RecPAIR/trained_unets/ShNestedUnet_mixed_brightnessAdj_Adam_DIceLoss_DeepSup_256px_cells.pth'
# net = NestedUNet(num_classes=1)
# saved_model = '/hdd/RecPAIR/UnetPlus_240px_channels.pth'

saved_net = torch.load(saved_model)
net.load_state_dict(saved_net['model_state_dict'])

net.cuda()

threshold = 0.5

sz = np.array(io.imread(ims[0]).shape).astype('int')

classifier = mmSegmentation(net, threshold, sz) #currently threshold is computed automatically by otsu algorightm
classifier.segment(segmentation_loader)

