from __future__ import print_function, division
import os
import torch
import re
import sys
from torch.utils.data import Dataset, DataLoader
from torchvision import transforms, utils
from supporting_functions.loadDataSets import mmDataSetSegment
import supporting_functions.customTransformations as ct
from supporting_functions import fileLoaders as fl
from supporting_functions.mmSegmentation import mmSegmentation
from supporting_functions.postProcessIms import cleanUpChannels
from supporting_functions.postProcessIms import par_watershed
import numpy as np
from skimage import io
from concurrent.futures import ProcessPoolExecutor
import time

#import unets
from networks.PraNetUnet import PraNetUnet, ShallowPraNetUnet
from networks.UnetPlusPlus2 import NestedUNet, UNet, ShallowNestedUNet

#first segment channels on the first image

#######################
#      CHANNELS
#######################

#command input
exp_dir = ''
for chunk in sys.argv[1:]:
    exp_dir += chunk + ' '
exp_dir = exp_dir[:-1]

print(exp_dir)
ims = sorted(fl.getFileList(exp_dir, chan_name='PreprocessedPhase', first_only=True))

save_dir = 'SegmentedChannels'
fl.makeDirs(exp_dir = exp_dir, save_dir = save_dir)
# seg_transformations = transforms.Compose([transforms.ToPILImage(),
#                                           ct.PILPadTo16(),
#                                           transforms.ToTensor(),
#                                           ct.SegmentationNormalize()])
seg_transformations = transforms.Compose([ct.PadTo16(),
                                          transforms.ToTensor(),
                                          ct.SegmentationNormalize()])

ims_to_segment = mmDataSetSegment(phase_ims=ims,
                                  transform=seg_transformations, 
                                  output_folder=save_dir)

segmentation_loader = DataLoader(ims_to_segment, batch_size=1, shuffle=False, num_workers=16)
# net = ShallowNestedUNet(deep_supervision=False)
# saved_model = '/home/skynet/code/trained_networks/ShNestedUnet_mixed_brightnessAdj_Adam__HybridLoss_256px_channels.pth'

#changed nested unet into regular unet retrained on 01.06.2021
net = UNet(num_classes=1)
saved_model = '/hdd/RecPAIR/trained_unets/Unet_mixed_brightnessAdj_Adam_HybridLoss_512px_Channels.pth'

saved_net = torch.load(saved_model)
net.load_state_dict(saved_net['model_state_dict'])

net.cuda()
sz = np.array(io.imread(ims[0]).shape).astype('int')
classifier = mmSegmentation(net, sz)
classifier.segment(segmentation_loader)

ims_channels = sorted(fl.getFileList(exp_dir, chan_name='SegmentedChannels', first_only=True))
for im_path in ims_channels:
    cleanUpChannels(im_path)


#######################
#       CELLS
#######################

ims = sorted(fl.getFileList(exp_dir, chan_name='PreprocessedPhase', first_only=False))

save_dir = 'SegmentedPhase'

fl.makeDirs(exp_dir = exp_dir, save_dir = save_dir)

# seg_transformations = transforms.Compose([transforms.ToPILImage(),
#                                           ct.PILPadTo16(),
#                                           transforms.ToTensor(),
#                                           ct.SegmentationNormalize()])
seg_transformations = transforms.Compose([ct.PadTo16(),
                                          transforms.ToTensor(),
                                          ct.SegmentationNormalize()])

ims_to_segment = mmDataSetSegment(phase_ims=ims,
                                  transform=seg_transformations, 
                                  output_folder=save_dir)

segmentation_loader = DataLoader(ims_to_segment, batch_size=1, shuffle=False, num_workers=16)

# net = ShallowNestedUNet(deep_supervision=True)
# saved_model = '/home/skynet/code/trained_networks/ShNestedUnet_mixed_brightnessAdj_Adam_DeepSup_HybridLoss_256px_cells.pth'

# 2021-04-28 swapped the model for a 'normal' UNet trained with weights for touching cells. Should not need the watershed
net = UNet(num_classes=1)
# saved_model = '/hdd/RecPAIR/Unet_mixed_brightnessAdj_Adam_HybridLoss_512px_weighted.pth'
saved_model = '/hdd/RecPAIR/trained_unets/Unet_mixed_brightnessAdj_Adam_HybridLoss_512px_cellsUnweighted.pth' #01.06.2021
saved_net = torch.load(saved_model)
net.load_state_dict(saved_net['model_state_dict'])

net.cuda()

sz = np.array(io.imread(ims[0]).shape).astype('int')

classifier = mmSegmentation(net, sz)
classifier.segment(segmentation_loader)

# print('Starting watersheding.')
# start = time.time()

# #add watersheding
ims_pred = sorted(fl.getFileList(exp_dir, chan_name='SegmentedPhase', first_only=False))

# for im_pred in ims_pred:
#     par_watershed(im_pred)
# with ProcessPoolExecutor() as executor:
#    executor.map(par_watershed,ims_pred)
# end = time.time()
# print(f"Watersheding took {end-start} s.")
