import torch
#from torch.autograd import Variable
#import torch.nn.functional as F
from torch.utils.data import DataLoader
#import numpy as np

from skimage import io, measure, morphology

class mmSegmentation:
    
    """ trainer for microscopy experiment UNet. 
    The classifier will run 1 epoch, then test the accuracy on test data."""
    
    def __init__(self,net,sz):
        """
        The classifier for mm experiment
        """
        self.net = net
        self.use_cuda = torch.cuda.is_available()
        self.sz = sz

    def segment(self, seg_loader: DataLoader):

        self.net.eval()

        for sample in seg_loader:
            
            im, out_path = sample['im'], sample['path_to_save']

            im = im.cuda()

            pred = self.net(im)

            if len(pred)>1: #if deep supervision, use final prediction
                pred=pred[-1]

            pred = torch.sigmoid(pred)

            pred = pred.to('cpu').detach().numpy().squeeze(0).squeeze(0)

            pred = pred[0:self.sz[0], 0:self.sz[1]]
            
            thresh = 0.5
            pred = pred > thresh

            pred = morphology.remove_small_objects(pred,100)
            pred_labels = measure.label(pred).astype('uint16')
            io.imsave(out_path[0],pred_labels,compress=6,check_contrast=False)
