import numpy as np
from skimage import io, segmentation, morphology, measure, feature
from scipy import ndimage as ndi

from matplotlib import pyplot as plt

def cleanUpChannels(im_path):
    """clean up channel segmentation map so no barcodes are present.
       Selection to remove barcodes is done now automatically, can be improved.
       Now it is assumed that regions in segmented images are either large channels, or small noise, 
       and that channel masks are in 'one piece'."""
    
    im = io.imread(im_path)
    
    props = measure.regionprops(im)
    
    areas = [p['area'] for p in props]
    
    mean_size = np.mean(areas)
    
    im = morphology.remove_small_objects(im, mean_size / 5) #divide by 5 just to be sure
    
    #im = morphology.binary_dilation(im)
    
    im = measure.label(im).astype('uint16')
    
    #im_path2 = im_path.replace('.tiff', '_cleanedUp.tiff')
    
    io.imsave(im_path, im, compress=6, check_contrast=False)


#add here a code to remove cells crossing the most outer part of the channel?

def par_watershed(im_path):

    """ apply wathershed segmentation to images from NestedUnet and ovewrite old image"""

    label_im = io.imread(im_path)

    #for explanatin of watershed segmentation see:
    #1)https://scikit-image.org/docs/dev/auto_examples/segmentation/plot_watershed.html
    #2)https://blogs.mathworks.com/steve/2013/11/19/watershed-transform-question-from-tech-support/
    bin_im = label_im > 0

    distance = ndi.distance_transform_edt(bin_im)
    #define connectivity kernel for 2d image

    #connectivity size impacts some artifacts.
    conn = np.ones((5,5))

    h = 2 # h for h minimatransform
    #use morphological reconstruction, simillar to the matlab example
    distance2 = morphology.reconstruction(distance-h,distance)
    local_maxi = feature.peak_local_max(distance2, indices=False, footprint=conn)
    markers = measure.label(local_maxi)
    watershed_labels = segmentation.watershed(-distance, markers, mask=bin_im, watershed_line=True)
    watershed_labels = watershed_labels.astype('uint16')
    
    # plt.imshow(bin_im.astype('int') - (watershed_labels>0).astype('int'))
    # plt.show()
    # plt.imshow((watershed_labels>0).astype('int'))
    # plt.show()

    io.imsave(im_path, watershed_labels, compress=6, check_contrast=False)