from __future__ import print_function, division
import os
import torch
import re
from skimage import io, transform, filters
import numpy as np
import matplotlib.pyplot as plt
from torch.utils.data import Dataset, DataLoader
from torchvision import transforms, utils
from supporting_functions.loadDataSets import mmDataSetTrain
import supporting_functions.customTransformations as ct
from supporting_functions.loadFiles import getFileList
from supporting_functions.mmClassifier import mmClassifier

#import unets
from networks.PraNetUnet import PraNetUnet, ShallowPraNetUnet
from networks.UnetPlusPlus2 import NestedUNet, UNet, ShallowNestedUNet

# Ignore warnings
import warnings
warnings.filterwarnings("ignore")

def TrainWholeNet(save_name):

    #function to train 2 networks over night. Bad way to look for hyperparameters, but will do for now
    
    # images for training for channels
    train_what = 'channels'

    if train_what == 'channels':
        phase_dir = '/hdd/RecPAIR/Microscopy/EXP-19-BQ2462 neural net el665_el1544 P1/BQ2485_channels/1_neuralNetChannels/'
        phase_ims = sorted(getFileList(phase_dir, chan='aphase'))
        phase_dir2 = '/hdd/RecPAIR/Microscopy/EXP-19-BQ2462 neural net el665_el1544 P1/BQ2490_chans/phase/'
        phase_ims2 = sorted(getFileList(phase_dir2, chan=''))
        phase_ims = phase_ims + phase_ims2

        print(len(phase_ims))
        
        mask_dir = '/hdd/RecPAIR/Microscopy/EXP-19-BQ2462 neural net el665_el1544 P1/BQ2485_channels/analysis_neuralNet/'
        mask_ims = sorted(getFileList(mask_dir, chan=''))
        mask_dir2 = '/hdd/RecPAIR/Microscopy/EXP-19-BQ2462 neural net el665_el1544 P1/BQ2490_chans/mask/'
        mask_ims2= sorted(getFileList(mask_dir2,chan=''))
        mask_ims =mask_ims + mask_ims2
        print(len(mask_ims))
        
        
    if train_what == 'cells':
        #images for trainign for cells
        phase_dir_my = "/hdd/RecPAIR/Microscopy/EXP-19-BQ2462 neural net el665_el1544 P1/run3_20191005/"
        phase_ims_my = sorted(getFileList(phase_dir_my, chan='cbright'))
        phase_praneeth = '/hdd/RecPAIR/Microscopy/EXP-19-BQ2462 neural net el665_el1544 P1/Praneeth/'
        phase_ims_p = sorted(getFileList(phase_praneeth, chan='phase_crop'))
        phase_ims = phase_ims_my + phase_ims_p

        mask_dir_my = "/hdd/RecPAIR/Microscopy/EXP-19-BQ2462 neural net el665_el1544 P1/run3_gt/"
        mask_ims_my = sorted(getFileList(mask_dir_my,chan = ''))
        mask_praneeth = '/hdd/RecPAIR/Microscopy/EXP-19-BQ2462 neural net el665_el1544 P1/Praneeth/'
        mask_ims_p = sorted(getFileList(mask_praneeth, chan='mask_crop'))
        mask_ims = mask_ims_my + mask_ims_p
        
        phase_ims = phase_ims[400:500]
        mask_ims = mask_ims[400:500]


    #set up custom trasnformations
    crop_window = 256

    composed_train = transforms.Compose([ct.RandomCrop(crop_window),
                                         ct.RandomFlip90(),
                                         ct.RandomStretch(),
                                         ct.RandomRotate(),
                                         ct.CustomToTensor(),
                                         ct.CustomNormalize()])
    
    composed_val = transforms.Compose([ct.RandomCrop(512),
                                       ct.CustomToTensor(),
                                       ct.CustomNormalize()])
    
    train_dataset = mmDataSetTrain(phase_ims = phase_ims, 
                                mask_ims  = mask_ims,
                                transform = composed_train)


    valid_dataset = mmDataSetTrain(phase_ims = phase_ims, 
                                mask_ims  = mask_ims,
                                transform = composed_val)

    #check if GPU is available device
    device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")

    #load unet to GPU
    # net = NestedUNet()
    net = ShallowNestedUNet(deep_supervision=False)

    saved_model='/hdd/RecPAIR/trained_unets/ShNestedUnet_mixed_brightnessAdj_Adam_BCE_256px_channels.pth'
    saved_net = torch.load(saved_model)
    net.load_state_dict(saved_net['model_state_dict'])

    net.cuda()

    #initialise PyToch dataloader
    train_loader = DataLoader(train_dataset, batch_size=8, shuffle=True, num_workers = 12)

    validation_loader = DataLoader(valid_dataset, batch_size=1, shuffle=True)

    #set up training parameters
    # optimizer = torch.optim.Adam(net.parameters(),  lr=0.0001)
    optimizer = torch.optim.Adam(net.parameters(), lr=0.001)
    scheduler = torch.optim.lr_scheduler.StepLR(optimizer, step_size = 10, gamma = 0.5)
    num_epochs = 30
    #TODO validation at each epoch, make validation dataset and write validation fucntion. Accuracy?

    #save_name = '/hdd/RecPAIR/NestedUnet_512px_cells.pth'

    classifier = mmClassifier(net, optimizer, scheduler, 0.9, num_epochs, save_name)
    classifier.train(train_loader, validation_loader)

TrainWholeNet(save_name='/hdd/RecPAIR/ShNestedUnet_mixed_brightnessAdj_Adam__HybridLoss_256px_channels.pth')
# TrainWholeNet(subtrBg=True, save_name='/hdd/RecPAIR/Unet_bgSubtrTrue_496px_cells.pth')