%script to wrap around the analysis of the whole microsfluidic experiment
%in one go. Work in progress. Will be using NestedUnet for the segmentation
%and running this code on my computer. So far this wrapper seems to process
%an experiment with a single click. Using multicore tracking. Adjusting the
%transformation matrix to correct for pixel shift in the Nikon 6.

% TO DO:  ADD FLUORESCENCE MEASUREMENTS
%         ADD CELL MEASUREMENTS, RIBS?

anal_dir = 'analysis';
exp_dir = '/hdd/RecPAIR/Microscopy/EXP-21-BV3236 death';
exp_folders = {'exp2'};

for exp = 1
    tstart = tic;
    vars.this_exp = exp_folders{exp};
    vars.phase_name = 'aphase';
    vars.fluo_names = {''};    
    vars.add_fluo_chans = {''};
    vars.trans_mat_file = '';
    vars.param_file_base = '/home/skynet/code/pipeline_bolts/ParameterFile_.txt';
    vars.code_dir = '/home/skynet/code/ImAnalysis/';
    vars.sourceDir = fullfile(exp_dir,exp_folders{exp});
    vars.outputDir = fullfile(exp_dir,anal_dir,exp_folders{exp});    
    vars.imposeTranformation = 0; %PAY ATTENTION TO THIS!
    %chek if spots are needed
    analyseExperiment(vars)
    tseg_end = datestr(seconds(toc(tstart)),'HH:MM:SS');
    disp(['experiment analysed in whooping: ' tseg_end])
end

function analyseExperiment(vars)
%wrapper to analyse the whole experiment
%1-Create directories and parameter file
%2-Set initial conditions
%3-Preprocess the files
%4-Segment with NestedUnet
%5-Modify parameter file and put cells in channels
%6-Track cells
%7-Adjust transformation matrix

%make analysis folder
checkMakeDir(vars.outputDir)

% 1 prepare parameter file
mod_fields = {'phaseChannelNames','fluoChannelNames','doPreprocessing'};
mod_vals = {vars.phase_name,vars.fluo_names, 1};
param_file = fullfile(vars.outputDir,['ParameterFile_' vars.this_exp '.txt']);
%run function
modParamFile(vars.param_file_base,mod_fields,mod_vals,param_file) %initial parameter file

%make init folder, get initial parameters
checkMakeDir(fullfile(vars.outputDir,'initrun'))
% setInitialParameters(vars.sourceDir, fullfile(vars.outputDir,'initrun'), param_file, 'SelectBarcode', 'manual', 'Rotation', 0);

%make prod folder and preprocess the images
param_file = fullfile(vars.outputDir,['ParameterFile_' vars.this_exp 'Init.txt']); %parameter file after firrst settings
checkMakeDir(fullfile(vars.outputDir,'prod'))
% processCells(vars.sourceDir, fullfile(vars.outputDir,'prod'), param_file, vars.code_dir)

% %run the segmentation
% disp('starting segmentation')
% tseg = tic;
% % fullfile(vars.outputDir,'prod/')
% % system_command = ['python3 ' '/home/skynet/code/pipeline_bolts/NestedUnet/segmentationPipeline.py ' fullfile(vars.outputDir,'prod/')  ' 1>NUL 2>NUL'];%nested Unet 
% system_command = ['python3 ' '/home/skynet/code/pipeline_bolts/Unet/segmentationPipeline.py ' fullfile(vars.outputDir,'prod/')  ' 1>NUL 2>NUL'];%normal Unet trained with weights
% system(system_command)
% tseg_end = datestr(seconds(toc(tseg)),'HH:MM:SS');
% disp(['segmentation finished in ' tseg_end])

%put cells into the channels, seems to work
num_growth_chans = linkCellsChannels(fullfile(vars.outputDir,'prod'));
mod_fields = {'nGrowthChannels'};
mod_vals   = {num2str(num_growth_chans)};
modParamFile(param_file,mod_fields,mod_vals,param_file); %miod the existing parameter file

%set the property 'doPreprocessing' and 'doSegmentation' to 2, 'doBlobProcessing' to 1
mod_fields = {'doPreprocessing', 'doSegmentation', 'doBlobProcessing'};
mod_vals   = {'2','2','1'};
modParamFile(param_file,mod_fields,mod_vals,param_file); %miod the existing parameter file
%do the tracking
processCells(vars.sourceDir, fullfile(vars.outputDir,'prod'), param_file, vars.code_dir)

%adjust tranformation matrices
if vars.imposeTranformation
imposeTranformationMatrix(fullfile(vars.outputDir,'prod'), 'fluo594', 'CFP')
% imposeTranformationMatrix(fullfile(vars.outputDir,'prod'), 'fluo635', 'cy5')
end

%add cell measurement?

%add fluorescence measurements
disp('starting fluo calculations')
tFluo = tic;
for c = 1:length(vars.add_fluo_chans)
addFluorescence(fullfile(vars.outputDir,'prod'),vars.add_fluo_chans{c}, 20)
end
tFluo_end = datestr(seconds(toc(tFluo)),'HH:MM:SS');
disp(['fluo added  in ' tFluo_end])

%copy entire ImAnalysis foder and pipeling_bolts to the analysis forlder
dirs = {'/home/skynet/code/ImAnalysis','/home/skynet/code/pipeline_bolts'};
copyPipeline(fullfile(vars.outputDir,'code_version'),dirs)

end

%--------------------------------------------------------------------------
%all the other functions 
%--------------------------------------------------------------------------

function modParamFile(param_file, mod_fields, mod_vals, save_name)
%open parameter file, modify it and save. 
%mod_fields - a cell of strings with fields to modify
%mod_vals - a cell of cells with strings to set at fields. Can be multiple 
%           strings in one cell.

fid = fopen(param_file);
pfile = [];
while (~feof(fid))
   ln = fgetl(fid);
   splt_ln = strsplit(ln);
   flag = find(strcmp(mod_fields, splt_ln{1}));
   if ~isempty(flag)
       %a bit of spaggetti line but matlab is stubborn with strings
       ln = char(join([splt_ln{1} '=' string(mod_vals{flag})]));
   end   
   pfile = horzcat(pfile,[ln '\n']);
end
fOut = fopen(save_name, 'w');
fprintf(fOut, pfile);
end

function checkMakeDir(dir_path)
%checks if there is folder, or makes it if not
    if exist(dir_path,'dir') ~= 7
        mkdir(dir_path)
    end
end

function nGrowthChannels = linkCellsChannels(exp_dir)
%function to put blobs in corresponding channels after NestedUnet
%segmnentation. 
%exp_dir - is the 'prod' analysis folder.
%SegmentedChannels folder from nested unet segmentation has to be there
%
%updated on 02-06-2021 - changed to median filtering of the channel
%locations to deal with the problem where edges of the channels were
%expanded and mutliple channels were connected.

% exp_dir = fullfile(exp_dir, 'prod');
expInfoObj_placeholder = load(fullfile(exp_dir, 'expInfoObj.mat'));
expInfoObj = expInfoObj_placeholder.expInfoObj;
pos_name = expInfoObj.positions;
%chan_im = 'SegmentedChannels/img_000000000_KubNet.tiff';

num_frames = length(expInfoObj.imRange{strcmp(expInfoObj.channels,'aphase')});

chan_mat = zeros(1, 1, length(pos_name));

yoffset = 10; %specify where the channels masks are starting in the y axis.
dilateFactor = 1; %how much channels will bloat
for p = 1 : length(pos_name)
    this_ims = dir(fullfile(exp_dir,char(pos_name(p)),'SegmentedChannels','*tiff'));
    this_im = imread(fullfile(this_ims(1).folder, this_ims(1).name));
   
    %find peaks using median filtering. Should be resistant to noise
    chanMask = [0 median(this_im>0) 0]; %push 0 into the mask to isolate edge channels
    pksDiff = diff(chanMask);
    chanLocs = [find(pksDiff==1)' find(pksDiff==-1)'];
    chanWidth = median(chanLocs(:,2)-chanLocs(:,1));    
    bbox = [chanLocs(:,1)-dilateFactor,...
                repmat(yoffset,size(chanLocs,1),1),...
                chanLocs(:,2)-chanLocs(:,1)+dilateFactor*2,...
                repmat(size(this_im,1) - yoffset, size(chanLocs,1),1)]; %dilateFactor may be an error if the channel location is at 1!!
    
    %%previous method using region props
    %props = regionprops(this_im, 'BoundingBox');
    %bbox = reshape([props.BoundingBox],4,length(props))';
    chan_mat(1:size(bbox,1), 1:size(bbox,2), p) = bbox;
    %expInfoObj.setChannelLocations(char(pos_l(p)), repmat(bbox, [1,1,num_frames]))
end

%2-update parameters to match the number of channels to pass the artument
%later to the expInfObj.

nGrowthChannels = size(chan_mat,1);
expInfoObj.setParameters('nGrowthChannels', size(chan_mat,1))

for p = 1 :  length(pos_name)
    expInfoObj.setChannelLocations(char(pos_name(p)), repmat(chan_mat(:,:,p), [1,1,num_frames]))
end

expInfoObj.saveExpInfo

end

function imposeTranformationMatrix(exp_dir, fluo_chan_name, filter_cube_name)
%function to correct the pixel shift in Nikon 6. Currently its tested and
%works for the mCherry images shot with CFP cube. Can also correct mcherry
%imgaes show with venus cube. Input is the 
%exp_dir - analysis prod directory,
%fluo_chan_name - is supposed to be mCherry channel name,
%filter_cube_name - points to the transformation matrix

eio = load(fullfile(exp_dir,'expInfoObj.mat'));
expInfoObj = eio.expInfoObj;
% this_fluo = find(strcmp(expInfoObj.channels,'fluo594'));
this_fluo = find(strcmp(expInfoObj.channels,fluo_chan_name));

if strcmp(filter_cube_name,'CFP')
%     tf = load('/home/skynet/code/pipeline_bolts/tform_mcherry_imaged_on_CFP.mat');
    tf = load('/home/skynet/code/pipeline_bolts/tform_mcherry_imaged_on_CFP_20201201.mat');
elseif strcmp(filter_cube_name,'venus')
    tf = load('/home/skynet/code/pipeline_bolts/tform_mcherry_imaged_on_venus.mat');
elseif strcmp(filter_cube_name,'cy5')
    tf = load('/home/skynet/code/pipeline_bolts/tform_SYTO60_imaged_on_CFP.mat');
end

tform = tf.tform;

for p = 1:length(expInfoObj.transfMatrices)
    this_t = expInfoObj.transfMatrices{p}{this_fluo};
    for fr = 1:size(this_t,3)
        this_t(1:2,1:2,fr) = tform.T(1:2,1:2);
        this_t(3,1:2,fr) = this_t(3,1:2,fr) + tform.T(3,1:2);
    end
    expInfoObj.setAllT(expInfoObj.positions{p}, expInfoObj.channels{this_fluo}, this_t);
end
expInfoObj.saveExpInfo()
end

function addFluorescence(exp_dir,fluo_chan_name, bg_window)
%add fluorescence to the tracked cells. 
%exp_dir - the 'prod' directory
%fluo_chan_name- name of fluorescence channel
%bg_window - if not empty will be used to subract background calculation

eIO = load(fullfile(exp_dir,'expInfoObj.mat'));
expInfoObj = eIO.expInfoObj;
for pi = 1:length(expInfoObj.positions)
    pos_name = expInfoObj.positions{pi};
    %using sigma parameter from Spartak addition
    if ~isempty(bg_window)
        computeCellFluorescence(expInfoObj, pos_name, fluo_chan_name, bg_window);
    else
        computeCellFluorescence(expInfoObj, pos_name, fluo_chan_name);
    end
end
end

function copyPipeline(anal_dir,dirs)
%function to copy image processing files into analysis directory
for d = dirs
    ending = strsplit(d{1},'/');
    ending = ending{end};
    status = copyfile(d{1}, fullfile(anal_dir,ending));
    if status == 0
        disp('the analysis folder couldnt be copied')
    end
end
end