function T = cellSelectFun(expInfoObj,cas9_ranges, SOS_channel,SOS_increase)
%function takes expInfoObj and ranges of positions with Cas9 constructs,
% all other positions are expected to be with dCas9. cas9_ranges is a 
% vector with numbers.
%
% input:
%   expInfoObj - pipeline data
%   cas9_ranges - vector with numbers of positions with cas9, all
%                 other positions will be dcas9.
%   SOS_channel - name of SOS reporter channel, typically fluo440
%   SOS_increase - number of fold increase in SOS fluorescence signal
% 
% outupt:
%   T = experiment position cell_id cas9_version SOS_response
%   is a matlab table with categorical columns of experiment and cas9
%   version. 


%initialise avriables
counter = 1;
data_out = cell(0,7); %experiment, position, cell id, cas9 version, SOS response, birth frame, lifetime
exp_name = split(expInfoObj.sourceFolderPath,'/');
exp_name = exp_name{end};
index_SOS = strcmp(expInfoObj.fluoChannelNames,SOS_channel);

for pcount = 1:size(expInfoObj.positions,2)
    
    pos = expInfoObj.positions(pcount);
    %print what we do
    pstring = ['Position ' num2str(pcount) '/' num2str(size(expInfoObj.positions,2)) '\n'];
    fprintf(pstring)
    tmp = load(fullfile(expInfoObj.outputFolderPath, pos{:}, 'trackedCells.mat'));
    mCells = tmp.mCells;
    mFrames = tmp.mFrames;
    for ic = 1:length(mCells)
        if mCells(ic).isBadCell ~= 0, continue, end
        if isempty(mCells(ic).descendants), continue, end
        %             if mCells(ic).lifeTime < 15, continue, end
        
        life_time = mCells(ic).lifeTime;
        bday = mCells(ic).birthFrame;
        
        if any(cas9_ranges==pcount)
            cas9_version = "cas9";
        else
            cas9_version = "dcas9";
        end
        
        f = mCells(ic).fluoIntensities(index_SOS,:);
        t = mCells(ic).birthFrame:mCells(ic).lastFrame;
        idx = find(~isnan(f));
        [max_v,max_i] = max(f);
        [min_v,min_i] = min(f);
        selector = max_v > SOS_increase * min_v & max_i > min_i & max_v > 0.5;
        if selector
            stress_level = 1;
        else
            stress_level = 0;
        end
        
        tOut(counter,:) = {exp_name pcount ic cas9_version stress_level bday life_time};
        counter = counter + 1;
    end
    
end
T = cell2table(tOut,'VariableNames',{'exp','position','cellId','cas9','SOS','bday','lifetime'});
T.exp = categorical(T.exp);
T.cas9 = categorical(T.cas9);
end
