function [canvas, xy] = grabCellFluos(thisCell, expInfoObj, pos, im_dir, target_im_size, chan_name,crop_to_mask)
    %gets fluorescence slices for a cell. To be used with
    %"drawStressedCells" function. Makes a 'canvas' image of a cell lineage.
    
    ims = dir(fullfile(im_dir,'*.tiff'));
    life_time_phase = thisCell.birthFrame:thisCell.lastFrame;
    fluo_images = expInfoObj.getRange(pos, chan_name);
    fluo_indeces = expInfoObj.getIndices(pos, chan_name);
%     life_time = intersect(fluo_indeces,life_time_phase);
    thisPos = find(strcmp(expInfoObj.positions, pos));
    canvas = [];
    xy = [];
    fluos = cell(1,length(life_time_phase)); %cell to save fluo data
    inf_flag = 0;
    fat_matrix = [-2 -2 +2 +2]'; %use to enlarge the bouding box
    
    for each_slice = 1:length(life_time_phase)
        this_phase_frame = life_time_phase(each_slice);
        this_im_pic = find(fluo_indeces==this_phase_frame);
        bbox_index = each_slice;
        bbox = double(thisCell.boundingBoxes(:,bbox_index)) + fat_matrix;
        if ~isempty(this_im_pic)
            trans_mat =  expInfoObj.getTByFrame(pos,this_im_pic,chan_name);
            tform = affine2d(trans_mat); 
            this_fluo_im = imread(fullfile(im_dir, ims(this_im_pic).name));
            [this_fluo_im_adj,~] = imwarp(this_fluo_im,tform,'OutputView',imref2d(target_im_size));
            %subtract background
%             this_fluo_im_adj = this_fluo_im_adj - imgaussfilt(this_fluo_im_adj,20);
            cell_im = this_fluo_im_adj(bbox(2):bbox(2)+bbox(4), bbox(1):bbox(1)+bbox(3)); %make crop box fat
            if crop_to_mask %make it pretty?
                bb = thisCell.boundaries{each_slice} - fat_matrix(1:2)'; %minus because it moves in other direction to bbox
                szy = length(bbox(2):bbox(2)+bbox(4));
                szx = length(bbox(1):bbox(1)+bbox(3));
                cm = poly2mask(bb(:,1),bb(:,2),szy,szx);
                cm = imdilate(cm, strel('disk',2));
                cell_im(~cm) = 0;
            end
        else
           inf_flag = 1;
           bb = thisCell.boundaries{each_slice} - fat_matrix(1:2)';
           szy = length(bbox(2):bbox(2)+bbox(4));
           szx = length(bbox(1):bbox(1)+bbox(3));
           cell_im = zeros(szy,szx);
           indx = sub2ind(size(cell_im),bb(:,2),bb(:,1));
           cell_im(indx) = inf;
%             cm = poly2mask(bb(:,1),bb(:,2),szy,szx);
%             cell_im = cm.*inf;
        end
        fluos{each_slice} = cell_im;
    end
    %center the cell in the middle. Asusumption is that the cell is
    %straight. if it lies flat it will fail, but then do some math if we
    %arrive into that problem
    y_sz = cellfun(@(x) size(x,1),fluos);
    x_sz = cellfun(@(x) size(x,2),fluos);
    y_offsets = round((max(y_sz)-y_sz)/2);
    x_offset = 0;
    canvas = zeros(max(y_sz),sum(x_sz));
    for f = 1:length(fluos)
        canvas(y_offsets(f)+1:y_offsets(f)+y_sz(f),...
               x_offset+1:x_offset+x_sz(f)) = fluos{f};
           x_offset = x_offset+x_sz(f);
    end
    if inf_flag
        canvas(canvas==inf) = mean(canvas(canvas~=inf & canvas>0));
    end
    canvas = (canvas./max(canvas(:))).*2^8; %swith off if the image was normalised before