function detectSpotsSingleCell(expInfoObj, channel_name, save_hndl)
% function parses expInfoObj. Looks for spots and puts them in cell.
%
% inputs:
%   expInfoObj    - pipeline reference
%   channel_names - cell sturcture with the name of fluorescenc channels to
%                   proicess, minumum one channel
% outputs:
%   mCells with updated 'particles' field. 1 - frame s(correspondint to phase indexes), 2 - x coordinate in image space, 3 - y coordinate in image space.  
%                       ! important ! fluorescent image is adjusted to map
%                       on the phase contrast image
% kuba

for p = 1:size(expInfoObj.positions,2)
    
    fprintf(['position ' num2str(p) '/' num2str(size(expInfoObj.positions,2)) '\n'])
    
    this_pos = expInfoObj.positions{p};
        
    %define save file for tracked cells and prepare the folder
    chunks = split(expInfoObj.outputFolderPath,'/');
    save_name = fullfile(save_hndl,chunks{end-1},this_pos,['trackedCells.mat']);
    mkdir(fullfile(save_hndl,chunks{end-1},this_pos))
    
    %load mCells
    mc = load(fullfile(expInfoObj.outputFolderPath,this_pos,'trackedCells.mat'));
    mCells = mc.mCells;
    clear mc
    
    %get all the cells that live in each frame
    cells_data = getCellsInFrames(expInfoObj, mCells,this_pos);
    
    %now open the image stack and apply tranformation to it.
    %1) find the target image size for image wrap
    seg_files = dir(fullfile(expInfoObj.outputFolderPath, this_pos, 'SegmentedPhase', '*.tif*'));
    seg_info = imfinfo(fullfile(seg_files(1).folder, seg_files(1).name));
    seg_size = [seg_info.Height seg_info.Width];
    
    %2) load and transform fluorescenct stack
    fluo_stack = loadFluoStackAndTransform(expInfoObj, this_pos, channel_name, seg_size);
    
    %run the spot detection
    spots = findSpots(fluo_stack, expInfoObj, channel_name, this_pos, mCells, cells_data);
    
    spots = vertcat(spots{:});

    %gather spot information into the cell list mCells
    for ci = 1:length(mCells)
        mCells(ci).particles = [];
        mCells(ci).particles = spots(spots(:,1)==ci,[2:4]);
    end
    %save mCell files into save directory and play with the plotting later
%     dt{p} = mCells; 
    mCells.saveMCells(save_name,mCells)
    
    clear fluo_stack
    clear mCells
end

    function spots = findSpots(imstack, expInfoObj, channel_name, this_pos, mCells, cells_data)
        %Function goes throught the stack of fluorescent images, looks for
        %the spots with radial symmetry algorigthm, connects spots to the
        %cells and returns some data. Still unfinished
        % inputs: 
        %    imstack - cell structure with images (after registrations)
        %    expInfoObj - experiment data
        %    channel_name - string with the name of the fluorescent channel
        %    this_pos - string with the name of the position
        %    mCells - pipeline output
        %    cells_data - cell structure with each cell being a time frame 
        %                 in the experiment, with information about cell id 
        %                 and internal frame coordinate at given timepoint
        % outputs: 
        %   spots - cell structure with the fluorescent frames with the
        %           information about position of the spots and cell ids.
        %           1-cell id, 2-frame in phase, 3-x position, 4-y position
        % kuba
        
        %todo: filter the spots based on median, or similar methodology.
        
        % get the cell life frames that correspond to fluorescence
        fluo_ind = expInfoObj.getIndices(this_pos,channel_name);
        
        radii = [2 3];
        alpha =  3;
        beta = 0;
        gaussKernRadFactor = 1.5;
        radius = 3;
        thresh = 0.4;
        subPixel = 0;
        
        %wavelet
        plane = 2;
        noiseThreshold = 3.5;
        minSpotArea = 0;
        maxAxesRatio = 1.5;
        
        spots = cell(length(imstack),0);
        ysize = size(imstack{1},1);
        xsize = size(imstack{1},2);
        
        parfor f = 1:length(imstack)
%         for f = 1:10:61%:length(imstack)
            %            figure
            %            imshow(imstack{f},[])
            %            hold on
            im = imstack{f};
            
            %use gaussian filtering!
            im_bg = im-imgaussfilt(im,10);
            this_f = fluo_ind(f); %

            %detect spots, set up the parameters
            
            %run spot detection

            seg_mask = zeros(size(im));
            for c = 1:length(cells_data{this_f})
                c_id = cells_data{this_f}(c,1);
                c_fr = cells_data{this_f}(c,2);
                bb = mCells(c_id).boundaries{c_fr};
                bbox = double(mCells(c_id).boundingBoxes(:,c_fr));
%                 bb = bb + double(mCells(c_id).boundingBoxes(1:2,c_fr))';
                szX = max(bb(:,1)) - min(bb(:,1))+1;
                szY = max(bb(:,2)) - min(bb(:,2))+1;
                cell_mask = poly2mask(bb(:,1),bb(:,2), bbox(4), bbox(3));
                seg_mask(bbox(2) : bbox(2) + bbox(4)-1, bbox(1) : bbox(1) + bbox(3)-1) = cell_mask;
            end
            seg_mask = logical(seg_mask);
            
%             figure
%             imshowpair(im,seg_mask)

%             spotMask = BuildTrajectories.SMT_spotDetect(im, 'Plane', plane, ...
%                             'actualPlaneNoise', 'noiseTH', noiseThreshold, 'segMask', seg_mask);
%             dotCoords = Spot.Detect.computeSpotCentroids(spotMask,im,minSpotArea,maxAxesRatio);
%             xx = dotCoords(:,1);
%             yy = dotCoords(:,2);
%            
            S = BuildTrajectories.fastradial(im, radii, alpha, beta, gaussKernRadFactor);
            [yy,xx] = BuildTrajectories.nonmaxsuppts(S, radius, thresh, subPixel); %gives position of the spots
            %plot all cells in the frame
            temp_spots = [];
% 
%             figure
%             imshow(imadjust(im),[])
%             hold on
            
            for c = 1:length(cells_data{this_f})
            
                c_id = cells_data{this_f}(c,1);
                c_fr = cells_data{this_f}(c,2);
                bb = mCells(c_id).boundaries{c_fr};
                
                %a bit slower but more 'organic' shapes
%                 bb = dilatePolygonM(bb,2);
                
                %a little bit faster mathematical expansion of polygon
                x = bb(:,1) - mean(bb(:,1));
                y = bb(:,2) - mean(bb(:,2));
                Ax = 1 + 2/(max(x)-min(x)); %set to expand for about 2 pixels
                if Ax == Inf
                    Ax = 1;
                end
                Ay = 1 + 2/(max(y)-min(y));
                if Ay == Inf
                    Ay = 1;
                end
                bb(:,1) = x.*Ax + mean(bb(:,1));
                bb(:,2) = y.*Ay + mean(bb(:,2));
                
                bb = bb + double(mCells(c_id).boundingBoxes(1:2,c_fr))';
                
%                 plot(bb(:,1),bb(:,2),'g')
                
                in = inpolygon(xx,yy,bb(:,1),bb(:,2));
                                
                %check each spot here if its above median level of the
                %cell.
                %add peak finder filter to remove spots in the cells with
                %no spots, but bright background
                if any(round(bb(:,2))>ysize)
                    bb(round(bb(:,2))>ysize,2) = ysize;
                end
                if any(round(bb(:,1))>xsize)
                    bb(round(bb(:,1))>xsize,1) = xsize;
                end
                if any(round(bb(:,2))<1)
                    bb(round(bb(:,2))<1,2) = 1;
                end
                if any(round(bb(:,1))<1)
                    bb(round(bb(:,1))<1,1) = 1;
                end
                
                cell_im = im_bg(round(min( bb(:,2)):max(bb(:,2))) ,round(min(bb(:,1)):max(bb(:,1))));
                cell_median = median(cell_im(:));
                          
                if sum(in)>0
                    spot_idx = find(in);
                    for spt = 1:length(spot_idx)
                        this_x = xx(spot_idx(spt));
                        this_y = yy(spot_idx(spt));
                        this_spot_sig = im_bg(this_y-1:this_y+1,this_x-1:this_x+1);
                        if max(this_spot_sig(:)) < cell_median * 3
                            in(spot_idx(spt)) = 0;
                        end
                    end
                end
                
%                 test = disp('test');
%                 c_sig = sum(cell_im-median(cell_im(:)),2);
%                 pk = peakfinder(c_sig, [], median(c_sig)*2.5, 10, 1, 0);
                

                if nnz(in) == 0 %|| nnz(in) > 0 && isempty(pk) %no spots detected
                    temp_spots = [temp_spots; [c_id this_f nan nan]];
                    %add nan to particles here
%                     plot(xx(in),yy(in),'go')
                else
                    temp_spots = [temp_spots; [repmat(c_id,[sum(in),1]) repmat(this_f,[sum(in),1]) xx(in) yy(in)]];
                    %add particles to the data here
%                     plot(xx(in),yy(in),'ro')
                end
                
                %here is a photo of a single cell. Should I use this to
                %select the spots with more confidence? Instead of global
                %selection? -pos1, cell 36 is a good example of what can
                %happen.
                %
                % TRY TO USE MEDIAN FILTER AND PEAK DETECTION, SINCE ALL
                % CELLS ARE ALIGNED IN THE MICROFLUIDIC CHANNELS ANYWAY.
                
%                 cell_im =  im(round( min( bb(:,2)):max(bb(:,2))) ,round(min(bb(:,1)):max(bb(:,1))));
%                 xx_cell = xx(in)-floor(min(bb(:,1)))+1;
%                 yy_cell = yy(in)-floor(min(bb(:,2)))+1;
%                 
%                 c_sig = sum(cell_im-median(cell_im(:)),2);
%                 pk = peakfinder(c_sig, [], median(c_sig)*2, 1, 1, 0);
%                 
%                 if ~isempty(pk)
% %                 if sum(in) ~= 0
%                     line(bb(:,1),bb(:,2),'Color',[0 1 0])
%                 else
%                     line(bb(:,1),bb(:,2),'Color',[1 0 0])
%                 end
            end
            
            %and now go over the spots and try to fit a gaussian to
            %determine good and bad spots?
            %2d gaussian distributioin
            %g = @(A,X) A(1)*exp( -((X(:,:,1)-A(2)).^2/(2*A(3)^2) + (X(:,:,2)-A(4)).^2/(2*A(5)^2)) );
%             for pk = 1:size(temp_spots,1)
%                 if isnan(temp_spots(pk,3)), continue, end
%                 tx = round(temp_spots(pk,3));
%                 ty = round(temp_spots(pk,4));
%                 ts = im(ty-3:ty+3,tx-3:tx+3);
%                 plot(mean(ts))
% %                 imshow(ts,[],'InitialMagnification',2000)
%                 drawnow
%             end
            figure
            imshow(im,[min(im(:)) max(im(:))])
            hold on
            plot(temp_spots(:,3),temp_spots(:,4),'gx')
            drawnow
            spots{f} = temp_spots;
        end
    end

    function cell_data = getCellsInFrames(expInfoObj,mCells,this_pos)
        % function opens mCell file and find the cells in every segmented
        % frame of the experiment, following 'aphase' name of the fluo.
        %   input:
        %       expInfoObj
        %       mCells - mCells structure for the position
        %  output:
        %       cell_data = cell structure with each cell as a frame index,
        %           each cell contains id of a cell in the frame, and index
        %           in a mCell structure for the given timeframe.
        % kuba
        
        num_frames = expInfoObj.imRange(strcmp(expInfoObj.positions,this_pos),strcmp(expInfoObj.channels,'aphase'));
        cell_data = cell(1,length(num_frames{1}));
        for c = 1:length(mCells)
            life_time = mCells(c).birthFrame : mCells(c).lastFrame;
            for t = 1:length(life_time)
                if ~isempty(mCells(c).boundaries{t})
                    cell_data{life_time(t)} = [cell_data{life_time(t)}; [mCells(c).id t]];
                end
            end
        end
    end

    function im_stack = loadFluoStackAndTransform(expInfoObj, this_pos_name, chan_name, target_im_size)
        %function opens fluorescent image stack and applies the
        %transformation to it to match the segmentation image. Result is
        %a cell array with tranformed slices.
        %
        % inputs:
        %   expInfoObj     - pipeline information for given experiment
        %   this_pos_name  - string with the name of position to be
        %                    analysed
        %   chan_name      - string with the name of fluorescent channel
        %                    to be analysed
        %   target_im_size - the size of the first segmentedPhase image in
        %                    the position in the experiment, for imwarp
        % output:
        %   im_stack - cell array with transformed fluorescent slices
        % kuba
        
        im_dir = fullfile(expInfoObj.sourceFolderPath, this_pos_name, chan_name);
        ims = dir(fullfile(im_dir,'*.tif*'));
        ims = fullfile({ims.folder},{ims.name}); %cell for better memory usage
        
        im_stack = cell(length(ims),0);
        %first load then transform, maybe this will be of any help, i dunno
        
        ims_stack = cell(length(ims),0);
        parfor im = 1:length(ims)
            %             t = Tiff(ims{im});
            %             ims_stack{im} = read(t);
            ims_stack{im} = imread(ims{im});
        end
        %if we want to subrtract the background we should do it here.
        parfor im = 1:length(ims_stack)%may be an error when more fluo images than phase
            trans_mat =  expInfoObj.getTByFrame(this_pos_name,im,chan_name);
            tform = affine2d(trans_mat);
            im_tmp = ims_stack{im};
            [this_fluo_im_adj,~] = imwarp(im_tmp,tform,'OutputView',imref2d(target_im_size));
            im_stack{im} = this_fluo_im_adj;
        end
    end

end