function [bbone, ribs, cshape] = findBackbone(cellshape)
%function to find backbone of a cell shape.
%hmmm...
% 
cellshapeSmooth = smoothColumns(cellshape);
ds = diff(cellshapeSmooth,1);
dist_from_vertex_to_vertex = hypot(ds(:,1), ds(:,2));
cumulative_dist_along_path = [0; cumsum(dist_from_vertex_to_vertex,1)];
step_len = 2;%step lenght in pixels
dist_steps = cumulative_dist_along_path(1) : step_len : cumulative_dist_along_path(end);
if dist_steps(end) ~= cumulative_dist_along_path(end)
    dist_steps = horzcat(dist_steps, cumulative_dist_along_path(end));
end
[~, inx] = unique(cumulative_dist_along_path);
points = interp1(cumulative_dist_along_path(inx), cellshapeSmooth(inx,:), dist_steps, 'spline');

x = points(:,1);
y = points(:,2);

warning('off','MATLAB:triangulation:PtsNotInTriWarnId')
warning('off','MATLAB:delaunay:DupPtsDelaunayWarnId')
[vxp,vyp] = voronoi(x,y);
warning('on','MATLAB:triangulation:PtsNotInTriWarnId')
warning('on','MATLAB:delaunay:DupPtsDelaunayWarnId')

in_pre = all(vxp>min(x) & vxp<max(x), 1) & all(vyp>min(y) & vyp<max(y), 1);
vxp = vxp(:,in_pre);
vyp = vyp(:,in_pre);

in = inpolygon(vxp,vyp,x,y);
in = all(in);

% find points that connect to only one other point
vx = vxp(:,in);
vy = vyp(:,in);

bbone = pruneBackbone(vx,vy); %in fucntion
%bbone = extrapolateSpine(bbone, [x y]);

%use spline to smooth the backbone. Downsample. Don't use the very ends of the backbone.
t = 1:size(bbone,1);
smoothFactor = 3;
tq = linspace(1,size(bbone,1),size(bbone,1)/smoothFactor);
xs = spline(t,bbone(:,1),tq);
ys = spline(t,bbone(:,2),tq);
bbone = [xs' ys'];
bbone = smoothColumns(bbone);
bbone = extrapolateSpine(bbone, [x y]);


%make equally spaced backbone
%https://blogs.mathworks.com/steve/2012/07/06/walking-along-a-path/
d = diff(bbone,1);
dist_from_vertex_to_vertex = hypot(d(:,1), d(:,2));
cumulative_dist_along_path = [0; cumsum(dist_from_vertex_to_vertex,1)];
%define number of ribs
% num_points = 10;
%dist_steps = linspace(0, cumulative_dist_along_path(end), num_points);
%use even steps
step_len = 1;%step lenght in pixels
dist_steps = cumulative_dist_along_path(1) : step_len : cumulative_dist_along_path(end);

%add end if was lost in dist_steps generation
if dist_steps(end) ~= cumulative_dist_along_path(end)
    dist_steps = horzcat(dist_steps, cumulative_dist_along_path(end));
end

[~, inx] = unique(cumulative_dist_along_path);
points = interp1(cumulative_dist_along_path(inx), bbone(inx,:), dist_steps);
points = smoothColumns(points);
points = smoothColumns(points);

bbone = points;

% find the ribs on smoothed outline.
ribs = findRibs(points,x,y);

% bbone = [mean(ribs(:,[1 3]),2) mean(ribs(:,[2 4]),2)];
cshape = [x y];

    function [out] = pruneBackbone(vx,vy)
        % cleans the vornoi backbone of a cell and prunes the branches untill
        % only one main branch remains.
        % Kuba
        [~,rep_r] = find(vx(1,:) == vx(2,:) & vy(1,:) == vy(2,:)); %clean up voronoi, some points are doubled
        vx(:,rep_r) = [];
        vy(:,rep_r) = [];
        uni = ones(size(vx));
        while true
            for p = 1:numel(vx)
                if sum(vx(:) == vx(p)) > 1
                    if sum(vy(vx(:) == vx(p)) == vy(p)) > 1
                        uni(p) = 0;
                    end
                end
            end
            [~,ir] = find(uni);
            
            vx(:,ir) = [];
            vy(:,ir) = [];
            
            if sum(uni(:)) <= 2
                break
            end
            uni = ones(size(vx));
        end
        
        
        out = [vx(:) vy(:)];
        out = unique(out, 'rows'); %return only single points
        
        %find a point that is the most distant from all other points
        dmat = zeros(size(out,1),1);
        
        for d = 1:length(dmat)
            dmat(d) = sum(hypot(out(:,1) - out(d,1), out(:,2) - out(d,2)));
        end
        
        [~, frst_pt] = max(dmat);
        h = hypot(out(:,1) - out(frst_pt,1),out(:,2) - out(frst_pt,2));
        [~, idx] = sort(h);
        out = out(idx,:);
        % out = smoothdata(out,1);
        out = smoothColumns(out);
        if size(out,1) > 15
            out = out(2:end-1,:);
        end
        
    end

    function nb = extrapolateSpine(b,c)
        % extarpolate bbone to reach the cell outline and return new backbone
        % b - backbone [x y]
        % c - cell outline in [x y]
        % extend first end of the backbone and find the closest intesection with the
        % cell edge
        %Kuba
        
        [xi1, yi1] = findCrossingPoint(b(1:2,1),b(1:2,2),c);
        [xi2, yi2] = findCrossingPoint(b(end-1:end,1),b(end-1:end,2),c);
        nb = zeros(size(b,1)+2,size(b,2));
        nb(1,:) = [xi1,yi1];
        nb(2:end-1,:) = b;
        nb(end,:) = [xi2,yi2];
        
        function [xi, yi] = findCrossingPoint(x,y,c)
            %helper function extrapolating the segment of line and finding the
            %intersection with cell boundary
            slope = diff(y) / diff(x);
            if abs(slope) == Inf %if the straight line
                seg_x = [x(1) x(1)];
                seg_y = [min(c(:,2))-1 max(c(:,2))+1];
            else
                b_val = y(1) - slope*x(1);
                seg_x = [min(c(:,1))-1 max(c(:,1))+1];
                seg_y = seg_x * slope + b_val;
            end
            
            %          [xi,yi] = intersections(c(:,1),c(:,2),seg_x,seg_y);
            xy1 = [c(:,1) c(:,2) [c(2:end,1);c(1,1)] [c(2:end,2);c(1,2)]];
            xy2 = [seg_x(1) seg_y(1) seg_x(2) seg_y(2)];
            [xi,yi] = lineSegmentIntersectFast(xy1,xy2);
            
            d = hypot(xi-x(1), yi-(y(1)));
            [~, min_d] = min(d);
            xi = xi(min_d);
            yi = yi(min_d);
        end
        
    end

    function ribs = findRibs(points,x,y)
        %find the ribs. x,y is the cell outline shape
        %kuba
        ribs = zeros(size(points,1),4);
        ribs(1,:) = [points(1,1) points(1,2) points(1,1) points(1,2)];
        ribs(end,:) = [points(end,1) points(end,2) points(end,1) points(end,2)]; 
        
        for ij = 2:size(points,1)-1
            slope =  (points(ij-1,2)-points(ij+1,2)) / (points(ij-1,1)-points(ij+1,1));
            slope = -1/round(slope,4);
            if abs(slope) == Inf
                xx = [points(ij,1) points(ij,1)];
                yy = [min(y)-1 max(y)+1]; %spread the points a little in case they dont overlap
            else
                b_val = points(ij,2) - slope*points(ij,1);
                xx = [min(x)-1 max(x)+1];
                yy = slope*xx + b_val;
            end
            %         [xi,yi] = intersections(x,y,xx,yy);
            xy1 = [x y [x(2:end); x(1)] [y(2:end); y(1)]];
            xy2 = [xx(1) yy(1) xx(2) yy(2)];
            [xi,yi] = lineSegmentIntersectFast(xy1,xy2); %fastest function, can be modified to accept polygon and a line
            
            distance_to_cell = hypot(xi-points(ij,1),yi-points(ij,2));
            [~,idx_dist] = sort(distance_to_cell,'ascend'); %i guess it's for the case when there are 3 poits?
            xx2 = xi(idx_dist(1:2));
            yy2 = yi(idx_dist(1:2));
            
            ribs(ij,:) = [xx2(1) yy2(1) xx2(2) yy2(2)];
        end
        
        %order ribs. For each segment find the middle point and see on
        %which side the point stays.
        for eachrib = 2:size(ribs,1)-1
            %find the ribs, define a line between them, and then if they
            %are in wrong order, flip them.
            rr = ribs(eachrib,:);
            midpoint_pre = [(ribs(eachrib-1,1) + ribs(eachrib-1,3))/2, (ribs(eachrib-1,2)+ribs(eachrib-1,4))/2];
            midpoint_past = [(ribs(eachrib+1,1) + ribs(eachrib+1,3))/2, (ribs(eachrib+1,2)+ribs(eachrib+1,4))/2];
            ir = 1;
            ribxy = rr([ir ir+1]);
            sgn = sign((midpoint_past(1)-midpoint_pre(1)) * (ribxy(2)-midpoint_pre(2)) - (midpoint_past(2) - midpoint_pre(2)) * (ribxy(1) - midpoint_pre(1)));
            if sgn < 0
                ribs(eachrib,:) = [ribs(eachrib,[3 4]) ribs(eachrib,[1 2])];
            end
        end
    end

    function [xx,yy] = lineSegmentIntersectFast(XY1,XY2)
        %mod to be faster; kuba.
        %LINESEGMENTINTERSECT Intersections of line segments.
        %   OUT = LINESEGMENTINTERSECT(XY1,XY2) finds the 2D Cartesian Coordinates of
        %   intersection points between the set of line segments given in XY1 and XY2.
        %
        %   XY1 and XY2 are N1x4 and N2x4 matrices. Rows correspond to line segments.
        %   Each row is of the form [x1 y1 x2 y2] where (x1,y1) is the start point and
        %   (x2,y2) is the end point of a line segment:
        %
        %                  Line Segment
        %       o--------------------------------o
        %       ^                                ^
        %    (x1,y1)                          (x2,y2)
        %
        %   OUT is a structure with fields:
        %
        %   'intAdjacencyMatrix' : N1xN2 indicator matrix where the entry (i,j) is 1 if
        %       line segments XY1(i,:) and XY2(j,:) intersect.
        %
        %   'intMatrixX' : N1xN2 matrix where the entry (i,j) is the X coordinate of the
        %       intersection point between line segments XY1(i,:) and XY2(j,:).
        %
        %   'intMatrixY' : N1xN2 matrix where the entry (i,j) is the Y coordinate of the
        %       intersection point between line segments XY1(i,:) and XY2(j,:).
        %
        %   'intNormalizedDistance1To2' : N1xN2 matrix where the (i,j) entry is the
        %       normalized distance from the start point of line segment XY1(i,:) to the
        %       intersection point with XY2(j,:).
        %
        %   'intNormalizedDistance2To1' : N1xN2 matrix where the (i,j) entry is the
        %       normalized distance from the start point of line segment XY1(j,:) to the
        %       intersection point with XY2(i,:).
        %
        %   'parAdjacencyMatrix' : N1xN2 indicator matrix where the (i,j) entry is 1 if
        %       line segments XY1(i,:) and XY2(j,:) are parallel.
        %
        %   'coincAdjacencyMatrix' : N1xN2 indicator matrix where the (i,j) entry is 1
        %       if line segments XY1(i,:) and XY2(j,:) are coincident.
        
        % Version: 1.00, April 03, 2010
        % Version: 1.10, April 10, 2010
        % Author:  U. Murat Erdem
        
        %%% Argument check.
        %-------------------------------------------------------------------------------
        %
        % validateattributes(XY1,{'numeric'},{'2d','finite'});
        % validateattributes(XY2,{'numeric'},{'2d','finite'});
        
        [n_rows_1,n_cols_1] = size(XY1);
        [n_rows_2,n_cols_2] = size(XY2);
        
        if n_cols_1 ~= 4 || n_cols_2 ~= 4
            error('Arguments must be a Nx4 matrices.');
        end
        
        %%% Prepare matrices for vectorized computation of line intersection points.
        %-------------------------------------------------------------------------------
        X1 = repmat(XY1(:,1),1,n_rows_2);
        X2 = repmat(XY1(:,3),1,n_rows_2);
        Y1 = repmat(XY1(:,2),1,n_rows_2);
        Y2 = repmat(XY1(:,4),1,n_rows_2);
        
        XY2 = XY2';
        
        X3 = repmat(XY2(1,:),n_rows_1,1);
        X4 = repmat(XY2(3,:),n_rows_1,1);
        Y3 = repmat(XY2(2,:),n_rows_1,1);
        Y4 = repmat(XY2(4,:),n_rows_1,1);
        
        X4_X3 = (X4-X3);
        Y1_Y3 = (Y1-Y3);
        Y4_Y3 = (Y4-Y3);
        X1_X3 = (X1-X3);
        X2_X1 = (X2-X1);
        Y2_Y1 = (Y2-Y1);
        
        numerator_a = X4_X3 .* Y1_Y3 - Y4_Y3 .* X1_X3;
        numerator_b = X2_X1 .* Y1_Y3 - Y2_Y1 .* X1_X3;
        denominator = Y4_Y3 .* X2_X1 - X4_X3 .* Y2_Y1;
        
        u_a = numerator_a ./ denominator;
        u_b = numerator_b ./ denominator;
        
        % Find the adjacency matrix A of intersecting lines.
        INT_X = X1+X2_X1.*u_a;
        INT_Y = Y1+Y2_Y1.*u_a;
        INT_B = (u_a >= 0) & (u_a <= 1) & (u_b >= 0) & (u_b <= 1);

        xx = INT_X(INT_B);
        yy = INT_Y(INT_B);
    end


    function mat = smoothColumns(mat)
        %smooth matrix by columns. Somehow matlab is not doing this right for this
        %application.
        %kuba
        for n = 1:size(mat,2)
            mat(:,n) = smooth(mat(:,n));
        end
    end
end