function T = getCellStatsPar(expInfoObj,cas9_ranges,SOS_channel,SOS_increase)
%function takes expInfoObj and ranges of positions with Cas9 constructs,
% all other positions are expected to be with dCas9. cas9_ranges is a 
% vector with numbers.
%
% input:
%   expInfoObj - pipeline data
%   cas9_ranges - vector with numbers of positions with cas9, all
%                 other positions will be dcas9.
%   SOS_channel - name of CFP sos channel, for exapmle 'flup440'
%   SOS_increase - fold increase in SOS to classify a cell as doing SOS

% outupt:
%   T - is a matlab table with categorical columns of experiment and cas9
%       version. 
%       fields : exp, position, cellId, cas9, birth, lifetime, SOS, times, areas, frames, CFP 


%initialise avriables
% data_out = cell(0,7); %experiment, position, cell id, cas9, times, areas, frames
exp_name = split(expInfoObj.sourceFolderPath,'/');
exp_name = exp_name{end};
index_SOS = strcmp(expInfoObj.fluoChannelNames,SOS_channel);

T_data = cell(size(expInfoObj.positions,2),1);
for pcount = 1:size(expInfoObj.positions,2)
    pos = expInfoObj.positions(pcount);
    %print what we do
    pstring = ['Position ' num2str(pcount) '/' num2str(size(expInfoObj.positions,2)) '\n'];
    fprintf(pstring)
    tmp = load(fullfile(expInfoObj.outputFolderPath, pos{:}, 'trackedCells.mat'));
    mCells = tmp.mCells;
    mFrames = tmp.mFrames;
    data_out = cell(length(mCells),0);
    counter = 1;
    for ic = 1:length(mCells)
        if mCells(ic).isBadCell ~= 0, continue, end
        if isempty(mCells(ic).descendants), continue, end
        %             if mCells(ic).lifeTime < 15, continue, end
        
        if ismember(pcount,cas9_ranges)
            cas9_version = "cas9";
        else
            cas9_version = "dcas9";
        end
        
        f = mCells(ic).fluoIntensities(index_SOS,:);
        t = mCells(ic).birthFrame:mCells(ic).lastFrame;
        idx = find(~isnan(f));
        [max_v,max_i] = max(f);
        [min_v,min_i] = min(f);
        selector = max_v > SOS_increase * min_v & max_i > min_i & max_v > 0.5;
        if selector
            stress_level = 1;
        else
            stress_level = 0;
        end
       
        good_frames = mCells(ic).badSegmentations == 0;
        times = mCells(ic).detectionTimes(good_frames);
        areas = mCells(ic).areas(good_frames);
        life_frames = mCells(ic).birthFrame:mCells(ic).lastFrame;
        life_frames = life_frames(good_frames);
        fluo_data = mCells(ic).fluoIntensities(index_SOS,good_frames);
        
        data_out{counter} = {exp_name
                            pcount 
                            ic
                            cas9_version
                            mCells(ic).birthFrame
                            mCells(ic).lifeTime
                            stress_level
                            times
                            areas
                            life_frames
                            fluo_data}; 
                        
        counter = counter + 1;
    end
    T_data{pcount} = data_out;
end

T_cell = horzcat(T_data{:});
T_cell = horzcat(T_cell{:})';
T = cell2table(T_cell,'VariableNames',{'exp','position','cellId','cas9','birth','lifetime','SOS','times','areas','frames','CFP'});% T.exp = categorical(T.exp);
T.exp = categorical(T.exp);
T.cas9 = categorical(T.cas9);
T.cellId = categorical(T.cellId);

end
