function T_out = selectInducedCells(expInfoObj,cas9_ranges,SOS_channel_name,SOS_fold_increase)
%function will parse the experiment and look for induced cells in every
%positions. It returns a table with position and id of each inducecd cell.
% part of 'selectInducedCellsGUI'

% input:
%   expInfoObj   - pipeline data
%   cas9_ranges  - vector with numbers of positions with cas9, all
%                  other positions will be dcas9.
%   SOS_channel_name  - name of CFP sos channel, for exapmle 'flup440'
%   SOS_fold_increase - fold increase in SOS to classify a cell as doing SOS

% outupt:
%   T -   is a structure holding data, and expInfoObj for a given
%         experiment
%   T.T - is a matlab structure with fields :
%          exp, position, cell, cas9_version,brith, lifetime, bounding boxes, outlines ,SOS
%   T.expInfoObj - pipeline information

exp_name = split(expInfoObj.sourceFolderPath,'/');
exp_name = exp_name{end};
index_SOS = strcmp(expInfoObj.fluoChannelNames,SOS_channel_name);

T_data = cell(size(expInfoObj.positions,2),1);
parfor pcount = 1:size(expInfoObj.positions,2)
% for pcount = 1:size(expInfoObj.positions,2)
    pos = expInfoObj.positions(pcount);
    %print what we do
%     pstring = ['Position ' num2str(pcount) '/' num2str(size(expInfoObj.positions,2)) '\n'];
%     fprintf(pstring)
    tmp = load(fullfile(expInfoObj.outputFolderPath, pos{:}, 'trackedCells.mat'));
    mCells = tmp.mCells;
    data_out = cell(length(mCells),0);
    counter = 1;
    for ic = 1:length(mCells)
        if mCells(ic).isBadCell ~= 0, continue, end
        %check if this parapeter will change anything
%         if tcll.areas(1) < 100, continue, end %important, ther should be a limit on cell areas!
        
        if isempty(mCells(ic).descendants), continue, end
%         if mCells(ic).lifeTime < 15, continue, end
        
        if ismember(pcount,cas9_ranges)
            cas9_version = "cas9";
        else
            cas9_version = "dcas9";
        end
        
        f = mCells(ic).fluoIntensities(index_SOS,:);
        f = f(~isnan(f));
        if length(f) < 3, continue, end
% %         f = smooth(f);
%         t = mCells(ic).birthFrame:mCells(ic).lastFrame;
%         idx = find(~isnan(f));


%         [max_v,max_i] = max(f);
%         [min_v,min_i] = min(f);
        
        %SOS selector
%         selector = max_v > SOS_fold_increase * min_v & max_i > min_i & max_v > 0.5;
        try
        selector = f(end) > SOS_fold_increase * f(1);
        catch
            disp('')
        end
        if selector
            stress_level = 1;
        else
            stress_level = 0;
        end
        
%         division = ~isempty(mCells(ic).descendants);
        
%         if stress_level
            data_out{counter} = {exp_name
                pcount
                ic
                cas9_version
                mCells(ic).birthFrame
                mCells(ic).lifeTime
                mCells(ic).boundingBoxes
                mCells(ic).boundaries
                stress_level};
            counter = counter + 1;
%            end
        
    end
    T_data{pcount} = data_out;
end
T_cell = horzcat(T_data{:});
T_cell = horzcat(T_cell{:})';
% T = cell2table(T_cell,'VariableNames',{'exp','position','cell','cas9','birthFrame','lifetime','boundingBoxes','outlines','SOS'});
T = cell2struct(T_cell,{'exp','position','cell','cas9','birthFrame','lifetime','boundingBoxes','outlines','SOS'},2);
% T.exp = categorical(T.exp);
% T.cas9 = categorical(T.cas9);
T_out.T = T;
T_out.expInfoObj = expInfoObj;
end