function T = getFluoSlices(T,channels_names)
% function parses 'T' data from the 'selectInducedCells' function. For each
% position  of the experiment it splices images and saves them into the
% structure 'T'.

% input:
%   T             - structure from 'selectInducedCells' function
%   channel_names - cell sturcture with the name of fluorescenc channels to
%                   proicess, minumum one channel
% output
%   T             - with new field fluo594 and fluo440; raw fluorescent
%                   slices for each cell, expanded by 2 pixels in each
%                   direction!

%todo: channel names should be importable

% part of 'selectInducedCellsGUI'

%load image stack for a single position

num_channels = length(channels_names);
fluo_data = cell(0,num_channels);
for p = 1:size(T.expInfoObj.positions,2)
% for p = 1
    
    this_p = T.expInfoObj.positions{p};
    
    %find the target image size for image wrap
    seg_files = dir(fullfile(T.expInfoObj.outputFolderPath, this_p, 'SegmentedPhase', '*.tif*'));
    seg_info = imfinfo(fullfile(seg_files(1).folder, seg_files(1).name));
    seg_size = [seg_info.Height seg_info.Width];
    
    %cells in given position
    cells_in_pos = find([T.T.position] == p);
    
    %load and transform fluorescenct stack
    fluo_stack = cell(num_channels,0);
    for chan_i = 1:num_channels
        fluo_stack{chan_i} = loadFluoStackAndTransform(T.expInfoObj, this_p, channels_names{chan_i}, seg_size);
    end

    disp(p)
    
    %process each cell
    tic
    for tc = 1:length(cells_in_pos)
        this_cell = T.T(cells_in_pos(tc));
        %and each channel
        for chan_c = 1:num_channels
            this_chan_name = channels_names{chan_c};
            fluo_tmp = getSingleCellFluos(T.expInfoObj,this_cell,this_chan_name,this_p,seg_size,fluo_stack{chan_c});
            fluo_data{cells_in_pos(tc),chan_c} = fluo_tmp;
        end
    end
    toc
end

for chan_c = 1:num_channels
    this_chan_name = channels_names{chan_c};
    [T.T.(this_chan_name)] =  fluo_data{:,chan_c};
end

    function fluos = getSingleCellFluos(expInfoObj, this_cell, chan_name, pos, target_im_size,fluo_stack)
        %function returns spliced fluorscent images for a single cell. Par loop
        %opens each image and does the croppint.
        %
        %  inputs:
        %    expInfoObj     - pipeline informations
        %    this_cell      - structure holding cell informations, from
        %                     'selectInducedCells' function.
        %    chan_name      - name of fluorescent channel to process
        %    pos            - character name of position (eg 'Pos10')
        %    target_im_size - size of 'SegmentedPhase' image for fluo im
        %                     transformations
        
%         im_dir = fullfile(expInfoObj.sourceFolderPath, pos, chan_name);
%         ims = dir(fullfile(im_dir,'*.tif*'));
%         ims = fullfile({ims.folder},{ims.name}); %cell for better memory usage
        life_time_phase = this_cell.birthFrame:this_cell.birthFrame+this_cell.lifetime-1;
        fluo_indeces = expInfoObj.getIndices(pos, chan_name);
        fluos = cell(1,length(life_time_phase)); %cell to save fluo data
        fat_matrix = [-2 -2 +2 +2]'; %use to enlarge the bouding box
        
        for each_slice = 1:length(life_time_phase)
%         parfor each_slice = 1:length(life_time_phase)
            this_phase_frame = life_time_phase(each_slice);
            this_im_pic = find(fluo_indeces==this_phase_frame);
            bbox_index = each_slice;
            bbox = double(this_cell.boundingBoxes(:,bbox_index)) + fat_matrix;
            if ~isempty(this_im_pic)
%                 trans_mat =  expInfoObj.getTByFrame(pos,this_im_pic,chan_name);
%                 tform = affine2d(trans_mat);
%                 this_fluo_im = imread(ims{this_im_pic});
                this_fluo_im = fluo_stack{this_im_pic};
                this_fluo_im_adj = this_fluo_im;
%                 [this_fluo_im_adj,~] = imwarp(this_fluo_im,tform,'OutputView',imref2d(target_im_size));
                %             this_fluo_im_adj = this_fluo_im_adj - imgaussfilt(this_fluo_im_adj,15);
                
                %make sure the image fits
                if(bbox(1)) < 1
                    bbox(1) = 1;
                end
                if(bbox(2)) < 1
                    bbox(2) = 1;
                end
                if bbox(2)+bbox(4) > target_im_size(1)
                    bbox(4) = target_im_size(1) - bbox(2);
                end
                if bbox(1)+bbox(3) > target_im_size(2)
                    bbox(3) = target_im_size(3) - bbox(1);
                end
                
                cell_im = this_fluo_im_adj(bbox(2):bbox(2)+bbox(4), bbox(1):bbox(1)+bbox(3)); %make crop box fat
                fluos{each_slice} = cell_im;
            else
                fluos{each_slice} = zeros(bbox(4)+1,bbox(3)+1);
            end
        end
    end

    function im_stack = loadFluoStackAndTransform(expInfoObj, this_pos_name, chan_name, target_im_size)
        
        im_dir = fullfile(expInfoObj.sourceFolderPath, this_pos_name, chan_name);
        ims = dir(fullfile(im_dir,'*.tif*'));
        ims = fullfile({ims.folder},{ims.name}); %cell for better memory usage
        
        im_stack = cell(length(ims),0);
        %first load then transform
        ims_stack = cell(length(ims),0);
        parfor im = 1:length(ims)
%             trans_mat =  expInfoObj.getTByFrame(this_pos_name,im,chan_name);
%             tform = affine2d(trans_mat);
            ims_stack{im} = imread(ims{im});
%             [this_fluo_im_adj,~] = imwarp(im_tmp,tform,'OutputView',imref2d(target_im_size));
%             im_stack{im} = this_fluo_im_adj;
        end
        
        parfor im = 1:length(ims_stack)
            trans_mat =  expInfoObj.getTByFrame(this_pos_name,im,chan_name);
            tform = affine2d(trans_mat);
            im_tmp = ims_stack{im};
            [this_fluo_im_adj,~] = imwarp(im_tmp,tform,'OutputView',imref2d(target_im_size));
            im_stack{im} = this_fluo_im_adj;
        end
    end

end