 function fluos = getSingleCellFluos(expInfoObj, this_cell, chan_name, pos, target_im_size, im_stack)
    %function returns spliced fluorscent images for a single cell. Par loop
    %opens each image and does the croppint.
    %
    %  inputs:
    %    expInfoObj     - pipeline informations
    %    this_cell      - structure holding cell informations, from 
    %                     'selectInducedCells' function.
    %    chan_name      - name of fluorescent channel to process
    %    pos            - character name of position (eg 'Pos10')
    %    target_im_size - size of 'SegmentedPhase' image for fluo im 
    %                     transformations

    im_dir = fullfile(expInfoObj.sourceFolderPath, pos, chan_name);
    ims = dir(fullfile(im_dir,'*.tif*'));
    life_time_phase = this_cell.birthFrame:this_cell.birthFrame+this_cell.lifetime-1;
    fluo_indeces = expInfoObj.getIndices(pos, chan_name);
    fluos = cell(1,length(life_time_phase)); %cell to save fluo data
    fat_matrix = [-2 -2 +2 +2]'; %use to enlarge the bouding box

    for each_slice = 1:length(life_time_phase)
        this_phase_frame = life_time_phase(each_slice);
        this_im_pic = find(fluo_indeces==this_phase_frame);
        bbox_index = each_slice;
        bbox = double(this_cell.boundingBoxes(:,bbox_index)) + fat_matrix;
        if ~isempty(this_im_pic)
            trans_mat =  expInfoObj.getTByFrame(pos,this_im_pic,chan_name);
            tform = affine2d(trans_mat);
            this_fluo_im = im_stack{this_im_pic};
%             this_fluo_im = imread(fullfile(im_dir, ims(this_im_pic).name));
            [this_fluo_im_adj,~] = imwarp(this_fluo_im,tform,'OutputView',imref2d(target_im_size));
            cell_im = this_fluo_im_adj(bbox(2):bbox(2)+bbox(4), bbox(1):bbox(1)+bbox(3)); %make crop box fat
            fluos{each_slice} = cell_im;
        else
            fluos{each_slice} = zeros(bbox(4)+1,bbox(3)+1);
        end
    end
 end